/**
 * Programmname: RoboCup-Kamikaze-Client <br>
 * Klasse: Actions <br>
 * <br>
 * Diese <code>Actions</code> Klasse ist zustndig fr komplexere Aktionen der Spieler,
 * die jede der Strategieklassen betreffen kann. Deshalb sind die am hufigsten 
 * genutzten Aktionen fr alle Strategieklassen hier zusammengefasst. 
 *
 * 
 *
 * @author Bernd Stieger
 * @version 1.0
 */
public class Actions {
	/**
	* das Weltmodellmodul des Clients
	*/
	private WorldModel wm;
	/**
	* das Actormodul des Clients
	*/
	private Actor actor;
	//Server-Parameter
	private double PLAYER_DECAY;
	private double BALL_DECAY;
	private double DASH_POWER_RATE;
	private double KICK_POWER_RATE;
	private double KICKABLE_MARGIN;
	//berrechnete Spielerrichtung, abhngig von den see-cycles
	private int calc_PlayerDir=0;
	private int calc_ViewDir=0;
	private double[] calc_PlayerPos = {0,0};


	/**
	* 
    * @param wm ein <code>WorldModel</code> Objekt
   	* @param a ein <code>Actor</code> Objekt
    */
	public Actions(WorldModel wm, Actor a) {
		this.wm=wm;
		this.actor = a;
		PLAYER_DECAY=ServerParameter.PLAYER_DECAY;
		BALL_DECAY=ServerParameter.BALL_DECAY;
		DASH_POWER_RATE=ServerParameter.DASH_POWER_RATE;
		KICK_POWER_RATE=ServerParameter.KICK_POWER_RATE;
		KICKABLE_MARGIN=ServerParameter.KICKABLE_MARGIN;
	}
	
	/**
	 * Erneuert die berrechneten Daten. Wenn die Daten des Weltmodels ankommen, whrend keine neue "see"-Nachricht
	 * seit dem letzten Kommando eingegangen ist, werden die aktuellen Daten, vom letzten Kommando,bernommen.
	 *
	 */
	public void refreshData() {
			calc_PlayerDir = wm.getBodyDirection();
			calc_PlayerPos= wm.getPlayerPos();
			calc_ViewDir= wm.getViewDirection();
		}
	
	/**
	* Spieler kann den Kopf in einem absoluten Winkel drehen
	* 
	* @param absAngle ein <code>int</code> Wert, der den absoluten Winkel enthaelt
	*	
	*/
	public void turn_neck(int absAngle) {
		int relAngle=0;
		//berechne relativen Winkel
		if (Math.abs(absAngle-calc_ViewDir)<=180)
			relAngle=absAngle-calc_ViewDir;
		else {
			
				if(calc_ViewDir>0)
					relAngle=360-(calc_ViewDir-absAngle);
				else
					relAngle=absAngle-calc_ViewDir-360;
			}
			
		
		actor.turn_neck(relAngle);
		
		}		
	/**
	 * Der Spieler dreht sich absolut in die gewnschte Richtung indem man die Koordinaten eingibt
	 *
	 * @param x ein <code>double</code> Wert, 
	 * @param y ein <code>double</code> Wert, <br>
	 * der den Winkel aus der Sicht der Koordinaten enthaelt im Format (x,y)
	 */
	public void turn(double x, double y) {
		double deltaX = Math.abs(calc_PlayerPos[0]-x);
		double deltaY = Math.abs(calc_PlayerPos[1]-y);
		double dist=Math.sqrt(deltaX*deltaX+deltaY*deltaY);
		int dir =0;
		
		
		if(x>=calc_PlayerPos[0]) {
			if(y>=calc_PlayerPos[1]) dir = (int)Math.toDegrees(Math.asin(deltaY/dist));
			if(y<calc_PlayerPos[1]) dir = (-1)*(int)Math.toDegrees(Math.asin(deltaY/dist));
		}
		if(x<calc_PlayerPos[0]) {
			if(y>calc_PlayerPos[1]) dir = 180-(int)Math.toDegrees(Math.asin(deltaY/dist));
			if(y<=calc_PlayerPos[1]) dir = (int)Math.toDegrees(Math.asin(deltaY/dist))-180;
			
		}

		turn(dir);


	}
	/**
	 * Der Spieler dreht sich absolut in die gewnschte Richtung
	 *
	 * @param absAngle ein <code>int</code> Wert, der den absoluten Winkel enthaelt
	 */
	public void turn(int absAngle) {
		int relAngle=0;
		
			if (Math.abs(absAngle-calc_PlayerDir)<=180) 
				relAngle=absAngle-calc_PlayerDir;
			else {
					if(calc_PlayerDir>0)
						relAngle=360-(calc_PlayerDir-absAngle);
					else
						relAngle=absAngle-calc_PlayerDir-360;
			}
			
			
		actor.turn(relAngle);
		
		calc_PlayerDir=absAngle;
		

	}
	/**
	 * dreht den Spieler um 90 nach rechts
	 */
	public void turn90() {
		if(Math.abs(calc_PlayerDir+90)<180)
			calc_PlayerDir+=90;
		else
			calc_PlayerDir=calc_PlayerDir+90-360;
		actor.turn(90);
		}
	/**
	 * 
	 * Spieler rennt zu der absoluten Position, die beliebig eingegeben werden kann
	 * @param x ein <code>double</code> Wert,
	 * @param y ein <code>double</code> Wert, <br>
	 * im Format (x,y), der die Position enthaelt
	 */
	public void run(double x, double y) {
		

		double deltaX = Math.max(calc_PlayerPos[0],x)-Math.min(calc_PlayerPos[0],x);
		double deltaY = Math.max(calc_PlayerPos[1],y)-Math.min(calc_PlayerPos[1],y);
		double dist=Math.sqrt(deltaX*deltaX+deltaY*deltaY);
		int dir =0;

		
		if(x>=calc_PlayerPos[0]) {
			if(y>=calc_PlayerPos[1]) dir = (int)Math.toDegrees(Math.asin(deltaY/dist));
			if(y<calc_PlayerPos[1]) dir = (-1)*(int)Math.toDegrees(Math.asin(deltaY/dist));
		}
		if(x<calc_PlayerPos[0]) {
			if(y>calc_PlayerPos[1]) dir = 180-(int)Math.toDegrees(Math.asin(deltaY/dist));
			if(y<=calc_PlayerPos[1]) dir = (int)Math.toDegrees(Math.asin(deltaY/dist))-180;
			
		}
		
		if((dist>3) && Math.abs(calc_PlayerDir-dir)>18) 
			turn(dir);
		if((dist>3) && Math.abs(calc_PlayerDir-dir)<=18) 
			dash(1.0);
		if((dist<=3) && (dist>1.0) && Math.abs(calc_PlayerDir-dir)>10) 
			turn(dir);
		if((dist<=3) && (dist>1.0) && Math.abs(calc_PlayerDir-dir)<=10) 
			dash(0.5);
		if((dist<=1.0) && (dist>0.6))
			dash(0.3);
	}
	
	/**
	 * Spieler schiesst bei Aufruf der Methode sofort aus allen Lagen Richtung gegnerisches Tor
	 *
	 */
	public void goalkick() {
		
		kick(100,wm.getOpGoalDirection());

			
	}
	
	/**
	 * Spieler kickt den Ball mit gewnschter Strke in eine absolute Richtung
	 *
	 * @param power ein <code>double</code> Wert,
	 * @param absAngle ein <code>int</code> Wert, <br>
	 * der Kraft und absolute Richtung des Schusses enthaelt
	 */
	public void kick(double power, int absAngle) {
		int relAngle=0;
		if (Math.abs(absAngle-calc_PlayerDir)<=180)
			relAngle=absAngle-calc_PlayerDir;
		else {	
					if(calc_PlayerDir>0)
						relAngle=360-(calc_PlayerDir-absAngle);
					else
						relAngle=absAngle-calc_PlayerDir-360;}
		actor.kick(power,relAngle);


			
	}
	/**
	 * Spieler kickt den Ball zu einer absoluten Position
	 *
	 * @param targetX ein <code>double</code> Wert,
	 * @param targetY ein <code>double</code> Wert, <br>
	 * der die Position angibt, wohin der Ball hingeschossen werden soll, enthaelt im Format (targetX,targetY).
	 */	
	public void kickToPos(double targetX, double targetY) {
		
		double deltaX = Math.abs(calc_PlayerPos[0]-targetX);
		double deltaY = Math.abs(calc_PlayerPos[1]-targetY);
		double dist=Math.sqrt(deltaX*deltaX+deltaY*deltaY);
		//absolute direction of target
		int dir =0;
		
		if(targetX>=calc_PlayerPos[0]) {
			if(targetY>=calc_PlayerPos[1]) dir = (int)Math.toDegrees(Math.asin(deltaY/dist));
			if(targetY<calc_PlayerPos[1]) dir = (-1)*(int)Math.toDegrees(Math.asin(deltaY/dist));
		}
		if(targetX<calc_PlayerPos[0]) {
			if(targetY>calc_PlayerPos[1]) dir = 180-(int)Math.toDegrees(Math.asin(deltaY/dist));
			if(targetY<=calc_PlayerPos[1]) dir = (int)Math.toDegrees(Math.asin(deltaY/dist))-180;
		}		

		int power=0;
		for (int i=1;i<100;i++) {
			double cDist=0; 
			double ep=i*KICK_POWER_RATE * (1- 0.25*(Math.abs(wm.getBallDirection())-Math.abs(wm.getViewDirection()))/180
								-0.25*(wm.getBallDistance()/KICKABLE_MARGIN));
			double cSpeed=wm.getBallSpeed()+ep;
			do { 
				cDist=cDist+cSpeed;
				cSpeed=cSpeed*BALL_DECAY;
			}
			while((dist>cDist)&&(cSpeed>0.1));
			if(cDist>dist) {power=i; break;}
		}

		kick(power,dir);
			
	}
	/**
	 * Spieler beschleunigt zu der jeweilig gewnschten Geschwindigkeit
	 *
	 * @param velocity ein <code>double</code> Wert, der die Geschwindigkeit enthaelt
	 */
	public void dash(double velocity) {
		double power=0;
		if(velocity>=wm.getVelocity()) {
			if((velocity-wm.getVelocity())/DASH_POWER_RATE>=100) 
				power=100;
				
			else
				power=(velocity-wm.getVelocity())/DASH_POWER_RATE;
		}
		if(velocity<wm.getVelocity()) 
			power=(wm.getVelocity()-velocity)/DASH_POWER_RATE;
			
		actor.dash(power);
		//predict player-position
		calc_PlayerPos[0]=calc_PlayerPos[0]+power*DASH_POWER_RATE*Math.cos(calc_PlayerDir);
		calc_PlayerPos[1]=calc_PlayerPos[1]+power*DASH_POWER_RATE*Math.sin(calc_PlayerDir);
	}
	/**
	 * Spieler dribbelt den Ball in Richtung gegnerisches Tor
	 * 
	 * 
	 */
	public void dribbleToOpGoal() {
			kick(80,wm.getOpGoalDirection());
	
					
		
		
	}
	
	
	
	
	

}