/**
 * Programmname: RoboCup-Kamikaze-Client
 *
 * Die <code>Actor</code> Klasse ist zustndig, fr die simplen Aktionen und Kommandos,
 * die jeder Spieler benutzen kann. Die Methoden verarbeiten die Kommandos und senden 
 * sie per communicationChannel Link
 * 
 *
 * @author Bernd Stieger
 * @version 1.0
 */


public class Actor
{
	
	
	/**
	* Referenz auf den communicationChannel
	*/
    private CommunicationChannel communicationChannel;
	
	
	/**
     * Erstellt eine neue <code>Actor</code> Instanz.
     *
     * @param communicationChannel ein <code>CommunicationChannel</code> Wert,der den 
     * communicationChannel Link zum Server herstellt,ber den die Kommandos versendet werden
     * @see CommunicationChannel
     */
   
    public Actor(CommunicationChannel communicationChannel) {
        this.communicationChannel = communicationChannel;
    }

    
	/**
	* Methode meldet den Spieler ab, indem ein String mithilfe vom communicationChannel Link gesendet wird
	*/
	public void bye() {
		communicationChannel.send("(bye)");
	}
    
	
	
	/**
	* Torwartfunktion: Der Torwart kann gegebenenfalls den Ball fangen
	*
	* @param direction eine <code>double</code> Wert, 
	* der den relativen Winkel vom Spieler zum Ball enthaelt
	*
	*/
    public void Catch(double direction)
    {
        communicationChannel.send(new String("(catch " + direction + ")"));
    }

    /**
     * Spieler rennt durch bergabe eines bestimmten Impulses
     *
     * @param power ein <code>double</code> Wert,der die Kraft des Impulses enthaelt
     * 
     */
    public void dash(double power)
    {
        communicationChannel.send(new String("(dash " + power + ")"));
    }

    /**
     * Spieler schiesst in eine gewnschte Richtung mit gewnschter Kraft
     *
     * @param power ein <code>double</code> Wert mit der entsprechenden Kraft
     * @param direction ein <code>double</code> Wert mit dem entsprechenden relativen Winkel
     */
    public void kick(double power, int direction)
    {
        communicationChannel.send(new String("(kick " + power + " " + direction + ")"));
    }


   	/**
     * Bringt den Spieler des Clients auf Feld. Nur mglich vor dem Anstoss und fr den Torwart,
     * nachdem er den Ball gefangen hat.
     *
     * @param x ein <code>double</code> Wert,
     * @param y ein <code>double</code> Wert,
     * der die absoluten Koordinaten enthaelt
     * 
     */
   
    public void move(double x, double y)
    {
        communicationChannel.send(new String("(move " + x + " " + y + ")"));
    }

    
    /**
    * Dreht den Spieler in die gewnschte Richtung
    *
    * @param moment ein <code>int</code> Wert, der den Gradwert enthaelt
    *
    */
    
    public void turn(int moment)
    {
        communicationChannel.send(new String("(turn " + moment + ")"));
    }

    //Turns the neck of the client on the field.
    /**
    * Dreht den Kopf des Spieler in die gewnschte Richtung
    *
    * @param angle ein <code>int</code> Wert, der den Gradwert enthaelt
    *
    */
    
    public void turn_neck(int angle)
    {
        communicationChannel.send(new String("(turn_neck " + angle + ")"));
    }

    //Sends messages to another players
    
    /**
    *
    * Methode sendet eine say-Nachricht ber den communicationChannel link
    *
    * @param message ein <code>String</code> Wert, der die Nachricht enthaelt
    */
       
    
    
    public void say(String message)
    {
        communicationChannel.send(new String("(say " + message + ")"));
    }

    //Changes the view
    
    /**
    *
    * ndert die Sicht des Spieler
    *
    * @param width ein <code>String</code> Wert
    * @param quality ein<code>String</code>) Wert,
    * der die Weite und die Qualitt der Sichtart enthaelt
    *
    */
    
    
    public void change_view(String width ,String quality)
    {
        communicationChannel.send(new String("(change_view " + width + " " + quality + ")"));
    }



}

