/** Programmname: RoboCup-Kamikaze-Client <br>
* Klasse: PlayerPerception
* <hr>
* Diese Klasse reprsentiert einen gesehenen Spieler im Weltmodell des Spielers
* <hr>
* * Standardwerte sind:
* <ul>
* <li> <code>int</code>- und <code>double</code>-Werte: 0 auer <code>lastUpdate</code>: -1
* </ul>
* @see WorldModel
* @author Michael Wagner
* @version 1.0 
*/
public class BallPerception {
	
	// absolute x
	private double absX = 0;
	// absolute y
	private double absY = 0;
	
	//recent x,y
	private double oldX = 0;
	private double oldY = 0;
	private double speed = 0;
	
	// time of last update
	
	private int lastUpdate =-1;
	private int relDirection= 0;
	private int direction = 0;
	private double distance = 0;
	
	/** ist Ball sichtbar?*/
	public boolean inSight = false;
	/** ist Ball auerhalb des Blickfeld des Spielers, aber innerhalb <code>visible_distance</code>?*/
	public boolean inVisibleDistance = false;
	
	/**
	* die absolute X-Koordinate
	* @return einen <code>double</code>-Wert zwischen -52.5 - 52.5
	*/
	public double absX() {return absX;}
	
	/**
	* die absolute Y-Koordinate 
	* @return einen <code>double</code>-Wert zwischen -34 - 34
	*/
	public double absY() {return absY;}
	/**
	 * die akuelle Geschwindgkeit
	 * @return <code>double</code>-Wert
	 */
	public double speed() {return speed;}
	
	/**
	* Zeitpunkt der letzten Aktualisierung
	* @return einen <code>int</code>-Wert
	*/
	public int lastUpdate() {return lastUpdate;}
	
	/**
	* absolute Richtung des Balls (vom Spieler aus gesehen)
	* @return einen <code>int</code>-Wert
	*/
	public int direction() {return direction;}
	/**
	* Enfernung vom Spieler
	* @return einen <code>double</code>-Wert
	*/
	public double distance() {return distance;}
		
	/**
	* relative Richtung des Balls (vom Spieler aus gesehen)
	* @return einen <code>int</code>-Wert
	*/
	public int relDirection() {return relDirection;}

	
	//update ball's location
	public void update(ObjectPerception raw_Ball,double[] playerPos,int absPlayerAngle, int time) {
    	//save old position
    	oldX=absX;
    	oldY=absY;
    	//set relative direction
        relDirection=raw_Ball.direction();               
        //set absolute direction
        if(Math.abs(absPlayerAngle+raw_Ball.direction())<=180)
        	direction = absPlayerAngle + raw_Ball.direction();
        else {if(absPlayerAngle>0)
        		direction = absPlayerAngle+raw_Ball.direction()-360;
        	if(absPlayerAngle<0)
        		direction = 360+(absPlayerAngle+raw_Ball.direction());
        }
        //distances from playerpos
        double deltaX, deltaY;
        distance = raw_Ball.distance();
        double c = Math.PI;
        double absBallPi = c*(direction/180.0);
        double wertcos = Math.cos(absBallPi);
        double wertsin = Math.sin(absBallPi);
        deltaX = raw_Ball.distance() * wertcos;
        deltaY = raw_Ball.distance() * wertsin;
        //set absolute postion
        absX = playerPos[0]+deltaX;
        absY = playerPos[1]+deltaY;
        //set speed
 		speed=Math.sqrt((absX-oldX)*(absX-oldX)+(absY-oldY)*(absY-oldY));
 		//set last update
		lastUpdate = time;
	}
	
	
	public double[] predictPos(int cycles) {
	
		double[] newPos = {absX,absY};
		double deltaX=0;
		double deltaY=0;
	
		deltaX = absX-oldX;
		deltaY = absY-oldY;
			
		int signDX=1;
		int signDY=1;
		if(deltaX<0) signDX=-1;
		if(deltaY<0) signDY=-1;
		double speed2=speed;
		if(speed2==0) return newPos;
		double angle=0;
		
		for(int i=0;i<cycles;i++) {
			speed2=speed2*ServerParameter.BALL_DECAY;
			if (deltaY!=0) angle = Math.abs(Math.atan(deltaX/deltaY));
			deltaX= speed2*Math.sin(angle)*signDX;
			deltaY= speed2*Math.cos(angle)*signDY;
			newPos[0]=newPos[0] + deltaX;
			newPos[1]=newPos[1] + deltaY;
		}
		
		return newPos;
	}
	
}