/** Programmname: RoboCup-Kamikaze-Client<br>
* Klasse: CommunicationChannel<br>
* <hr>
* Aufgabe: sendet Daten zum und empfngt Daten vom Server
* <hr>
*
* @author Markus Mllerpods/Andreas Lenz
* @version 1.0 / 4.7.02
*/

import java.io.*;
import java.net.*;

public class CommunicationChannel
{
	//Attribute
	private DatagramSocket socket;
	InetAddress serverAddress;
	private int serverPort;

	//Konstruktor
	/**
	* Erzeugt einen neuen Kommunikationskanal
	* @param serverName Die Adresse des Servers
	* @param serverPort Der Port des Servers
	*/
	public CommunicationChannel(String serverName, int serverPort)
	{
		try
		{
			// construct datagram socket and
			// bind it to any available port on the local machine
			this.socket        = new DatagramSocket();
			// set target address
			this.serverAddress = InetAddress.getByName(serverName);
			// set target port
			this.serverPort    = serverPort;
		}
		catch (SocketException se)
		{
		System.err.println("CommunicationChannel: Could not create datagram socket.");
		System.exit(1);
		}
		catch (UnknownHostException uhe)
		{
		System.err.println("CommunicationChannel: Could not determine the server's IP address.");
		System.exit(1);
		}
	}

	//Operationen
	/**
	* Sendet einen String zum Server
	*
	* @param message der String, der zum Server gesendet werden soll
	*/
	public void send(String message)
	{
		try
		{
			// convert the message to a byte array
			byte[] buffer = message.getBytes();
			// create the packet from the buffer and set the server's IP address and port
			DatagramPacket packet = new DatagramPacket(buffer, buffer.length, serverAddress, serverPort);
			// send the packet
			socket.send(packet);
		}
		catch (IOException ioe)
		{
			System.err.println("CommunicationChannel: Could not send datagram packet.");
		}
	}


	/**
	* Empfngt eine Nachricht vom Server
	*
	* @return der String der Nachrichten vom Server enthlt
	*/
	public String receive()
	{
		// allocate buffer for server message
		byte[] buffer = new byte[1024];
		// create datagram packet for receiving
		DatagramPacket packet = new DatagramPacket(buffer, buffer.length);
		try 
		{
			// receive packet, the method blocks until a datagram arrives
			socket.receive(packet);
		}
		catch (IOException ioe)
		{
			System.err.println("CommmunicationChannel: Could not receive datagram packet.");
		}
		// get the server port for the following actions
		serverPort = packet.getPort();
		// return the message as string
		return new String(packet.getData()).substring(0, packet.getLength()-1)+" ";  // cut off null characters & \n in buffer
	}
}