/** Programmname: RoboClient <br>
 * <br>
 * In dieser Klasse befindet sich die Strategie fr die Defensivspieler. Sie kann grob folgendermaen 
 * zusammengefat werden:
 * <ul>
 * <li> Spieler dreht sich, bis er den Ball sieht
 * <li> Wenn er der nchste Spieler des eigenen Teams zum Ball ist, rennt er hin und kickt ihn dann Richtung gegnerisches Tor
 * <li> Wenn ein anderer Spieler nher zum Ball steht, rennt er zu einer taktisch gnstigen Position
 * </ul>
 * @author Bernd Stieger/Michael Wagner
 * @version 1.0
 */
public class Defense {

	//comunication
	private CommunicationChannel communicationChannel;
	//world model
	private WorldModel wm;
	//actions
	private Actions actions;
	//acot
	private Actor actor;
	//playmode
	private int status=-1;
	/*  strategy=1 => look for ball
		strategy=2 => run to ball & act
		strategy=3 => run to strategic pos

		*/
	private int strategy=-1;
	//tactical position
	private double tacticalX=0;
	//distance to tactical position
	private double distToTacticalPos=-1;
	//flag for actuality of see-data
	private boolean wm_update=false;
	//ball postion
	private double ballX=0;
	private double ballY=0;
	//ball distance
	private double ballDist=0;
	//predicted ball position
	private double predX=0;
	private double predY=0;
	// distance to postion above
	private double distToPred=0;
	//flag whether player is closest player to position above
	private boolean closestToPred=false;
	
	/**
	 * @param wm	das Weltmodell des Spielers (Typ: <code>WorldModel</code>)
	 * @param com	der Kommunikationskannal zum Server (Typ: <code>CommunicationChannel</code>)
	 * @see WorldModel
	 * @see CommunicationChannel
	 */
	public Defense(WorldModel wm, CommunicationChannel com) {
		this.wm = wm;
		communicationChannel = com;
		actor = new Actor(com);
		actions = new Actions(wm,actor);
		
	}
	/**
	 * legt die aktuelle Strategie fest, berechnet dann eine gnstige Aktion und sendet diese an den Server 
	 * @param wm_update ein <code>boolean</code>-Wert, der aussagt, ob die vom Weltmodell bereitsgestellten Daten
	 *		  aktuell sind oder nicht (da "see"-Nachrichten	alle 150 ms, "sense_body" schon alle 100ms)
	 *	<br> <br>Strategiefindung: <br>
	 *	<ul>
	 *	<li> welcher Spielmodus liegt vor?
	 *	<li> berechne taktische Position
	 *	<li> wenn Spielmodus="play_on":
	 *		<ul>
	 *		<li> Wenn keine Strategie aus vorherigen Cycles vorhanden => berechne Strategie
	 *		<li> ansonsten bernehme Strategie aus vorherigen Cycles
	 *		</ul>
	 *	<li> 
	 *  </ul>
	 */	
	public void act(boolean wm_update) {
		this.wm_update=wm_update;
		// if see-data up-to-date refresh data in Actions-class, else predict data
		if(wm_update) {
			actions.refreshData();
			ballX = wm.getBallX();
			ballY = wm.getBallY();
			ballDist= wm.getBallDistance();}
		else {
			ballX = wm.getPredictedBallPos(1)[0];
			ballY = wm.getPredictedBallPos(1)[1];
			ballDist = Math.sqrt((ballX-wm.getPlayerPos()[0])*(ballX-wm.getPlayerPos()[0])+
								  (ballY-wm.getPlayerPos()[1])*(ballY-wm.getPlayerPos()[1]));
		}
		//set variables
		predX=wm.getPredictedBallPos(4)[0];
		predY=wm.getPredictedBallPos(4)[1];
		distToPred=Math.sqrt((wm.getPlayerPos()[0]-predX)*(wm.getPlayerPos()[0]-predX)+
							 (wm.getPlayerPos()[1]-predY)*(wm.getPlayerPos()[1]-predY));
		closestToPred=wm.isClosestPlayerToPredBall(4);
		// check playmode
		status = -1;
		
		if(wm.getPlayMode().equals("before_kick_off")) 					status = 0;
		if(wm.getPlayMode().startsWith("kick_off")) {
			if(wm.getPlayMode().equals("kick_off_"+wm.getSide())) 		status = 1;
			else														status = 1;
		}
		if(wm.getPlayMode().equals("play_on"))		   					status = 2;
		if(wm.getPlayMode().startsWith("kick_in")) {
			if(wm.getPlayMode().equals("kick_in_"+wm.getSide()))		status = 2;
			else														status = 11;
		}
		if(wm.getPlayMode().startsWith("free_kick")) {
			if(wm.getPlayMode().equals("free_kick_"+wm.getSide()))	 	status = 2;
			else														status = 11;
		}													
		if(wm.getPlayMode().startsWith("corner_kick")) {
			if(wm.getPlayMode().equals("corner_kick_"+wm.getSide())) 	status = 11;
			else														status = 3;
		}
		if(wm.getPlayMode().startsWith("goal")) 						status = 0;
		
		if(wm.getPlayMode().startsWith("goal_kick")) {
			if(wm.getPlayMode().equals("goal_kick_"+wm.getSide())) 		status = 11;
			else														status = 11;
		}
		
		
		if(wm.getPlayMode().startsWith("offside")) {
			if(wm.getPlayMode().equals("offside_"+wm.getSide()))		status = 11;
			else 									 					status = 11;
		}
		
		//set tactical position
		setTacticalX();
		//select playmode
		switch(status){
		
		
		//Before kick off
		case 0:	if(Math.abs(wm.getPlayerPos()[0]-wm.getStartX())>1 ||Math.abs(wm.getPlayerPos()[1]-wm.getStartY())>1)
					actor.move(wm.getStartX(),wm.getStartY());
				else			
					actions.turn(0,0);
				break;
		//kick off					
		case 1:		if((Math.abs(wm.getPlayerPos()[0]-wm.getStartX())>1) || (Math.abs(wm.getPlayerPos()[1]-wm.getStartY())>1))
						actor.move(wm.getStartX(),wm.getStartY());
					else
						actions.turn(0,0);
					break;
		
		//play on
		case 2:			
				if(strategy<1) {
					if(!wm.ballInSight()) {
						if(wm.ballInVisibleDistance()) 
							strategy=2;	
						else 
							strategy=1;
					}
					if(wm.ballInSight()) {
						if(ballX<10) {
							if(closestToPred || wm.isClosestPlayerToBall())
								strategy=2;
							else 
								strategy=3;
							}
						if(ballX>=10) 
							strategy=3;
						}
						
					}	
					
				
				play();
				break;
		
		
		//corner kick opponent		   
		case 3:		
		
		//go to tactical position
		case 11:	if(wm.getStamina()>1000) {
						if(distToTacticalPos>1) 
							actions.run(tacticalX,wm.getStartY());
						else {
							if(wm.ballInSight() && wm_update)
								actions.turn(wm.getBallDirection());
							else if(wm_update)
								actions.turn90();
							}	
						}
					break;
	
	
		}
	}
	
	//act based on strategy	
	private void play() {
		
		//which strategy?
		switch(strategy) {
			case 1:		//only turn if data is up-to-date
						if (wm_update) 
							actions.turn90(); 
						strategy=0; 
						break;
						
			case 2:		//cancel strategy?
						if((wm.getLastBallUpdate()>5)||(!closestToPred && 
								!wm.isClosestPlayerToBall())) 
							{strategy=0; break;}
						//run to ball & kick it to opponent goal
						if(ballDist>0.7) {
								if(distToPred>0.5)
									actions.run(predX,predY);
								else
									actions.turn(ballX,ballY);}
						else {
							actions.goalkick();
							strategy=0;}
						break;
						
			case 3:		//cancel strategy?
						if((wm.getLastBallUpdate()>10)||wm.ballInVisibleDistance()||
						    closestToPred ||wm.isClosestPlayerToBall())
							 {strategy=0; break;}
						//run to tactical position
						if(distToTacticalPos>1)
							actions.run(tacticalX,wm.getStartY());
						//turn to ball/search ball
						else {
							if(wm.ballInSight() && wm_update) 
								actions.turn(wm.getBallDirection());
							if(!wm.ballInSight() && wm_update) {
								actions.turn90();
								strategy=0;}
							}	
						break;
			
						
			}
		
		}
		
		//set tactical position
		private void setTacticalX() {
			if(ballX<=0)
				tacticalX=-35;
			else {
				if(ballX<=20)	
					tacticalX=-10;
				else {
					if(ballX>30)
						tacticalX=-10;
					}
				}
			if(status==11) 
				tacticalX=-25;
			if(status==3)
				tacticalX=-45;
				
			distToTacticalPos=Math.sqrt((wm.getPlayerPos()[0]-tacticalX)*(wm.getPlayerPos()[0]-tacticalX)+
									(wm.getPlayerPos()[1]-wm.getStartY())*(wm.getPlayerPos()[1]-wm.getStartY()));
						
					}	
}
		
		
	
