/** Programmname: RoboCup-Kamikaze-Client <br>
* Klasse: GoalPerception
* <hr>
* Diese Klasse reprsentiert das eigene bzw. gegnerische Tor im Weltmodell des Spielers
* <hr>
* @see WorldModel
* @author Michael Wagner
* @version 1.0 
*/

public class GoalPerception
{
	//Attribute
	private double distance = -1;
	private int direction = -181;
	private int rightPostDirection = -181;
	private double rightPostDistance = -1;
	private int leftPostDirection = -181;
	private double leftPostDistance = -1;
	//ownGoal?
	private boolean ownGoal = false;
	/** ist Tor sichtbar?*/
	public boolean inSight = false;

	//Konstruktor
	/**
	* @param ownGoal <code>true</code>, wenn eigenes Tor, ansonsten <code>false</code>
	*/	
	public GoalPerception(boolean ownGoal)
	{
		this.ownGoal=ownGoal;
	}

	//Operationen	
	/**
	* akualisiert die absolute Richtung des Tors
	* @param playerPos die absolute Spielerposition
	*/
	public void update(double[] playerPos)
	{
		double distX,distY;
		if(ownGoal)
		{
			distX = playerPos[0]+52.5;
			distY = Math.abs(playerPos[1]);
			distance=Math.sqrt(distX*distX+distY*distY);
			if(playerPos[1]>=0)
				direction=(int)Math.toDegrees(Math.asin(distY/distance))-180;
			else
				direction=180-(int)Math.toDegrees(Math.asin(distY/distance));
		}
		else
		{
			distX = 52.5-playerPos[0];
			distY = Math.abs(playerPos[1]);
			distance=Math.sqrt(distX*distX+distY*distY);
			if(playerPos[1]>=0)
				direction=(-1)*(int)Math.toDegrees(Math.asin(distY/distance));
			else
				direction=(int)Math.toDegrees(Math.asin(distY/distance));
		}
	}

	/**
	* Entfernung des Spieler zur Mitte des Tors
	* @return einen <code>double</code>-Wert
	*/
	public double distance()
	{
		return distance;
	}

	/**
	* die absolute Richtung zur Mitte des Tors
	* @return einen <code>double</code>-Wert
	*/
	public int direction()
	{
		return direction;
	}

	/**
	* Entfernung zum rechten Pfosten
	* @return einen <code>double</code>-Wert
	*/
	public double leftPostDistance()
	{
		return leftPostDistance;
	}

	/**
	* Entfernung zum linken Pfosten
	* @return einen <code>double</code>-Wert
	*/
	public double rightPostDistance()
	{
		return rightPostDistance;
	}
	
	/**
	* absolute Richtung des rechten Pfostens
	* @return einen <code>int</code>-Wert
	*/
	public int rightPostDirection()
	{
		return rightPostDirection;
	}
	/**
	* absolute Richtung des linken Pfostens
	* @return einen <code>int</code>-Wert
	*/
	public int leftPostDirection()
	{
		return leftPostDirection;
	}

}