/** Programmname: RoboClient <br>
 * <br>
 * In dieser Klasse befindet sich die Strategie fr den Torwart. Sie kann grob folgendermaen 
 * zusammengefat werden:
 * <ul>
 * <li> Torwart dreht sich, bis er den Ball sieht
 * <li> Wenn der Ball auerhalb des Strafraumes ist bzw. innerhalb der nchsten 3 Cycles nicht dorthin kommt, 
 *  	geht er ggf. zu seiner Ausgangsposition (-50,0) zurck und dreht er sich Richtung Ball 
 * <li> Wenn der Ball innerhalb des Strafraumes ist bzw.innerhlab der nchsten 3 Cycles dorthin kommt,
 *		rennt er zum Ball un versucht in zu catchen
 * <li> Falls das catchen erfolgreich war, kickt er den Ball sofort Richtung gegnerisches Tor
 * </ul>
 * <br> Bei eigenem Torabsto dreht sich der Torwart zum Ball und kickt ihn Richtung gegnerisches Tor
 
 * @author Bernd Stieger/Michael Wagner
 * @version 1.0
 */
 		
public class Goalie {

	//communication
	private CommunicationChannel communicationChannel;
	//goalie's world model
	private WorldModel wm;
	// actions
	private Actions actions;
	// basic actor
	private Actor actor;
	//playmode
	private int status=-1;
	/*  strategy=1 => goto tacticalPos
		strategy=2 => catch ball
		strategy=3 => search ball
		*/
	private int strategy=1;
	//tactical position (currently: -50/0)
	private double tacticalX=0;
	private double tacticalY=0;
	// distance to tactical position
	private double distToTacticalPos=-1;
	// flag for actuality of see-data
	private boolean wm_update=false;
	//ball postion
	private double ballX=0;
	private double ballY=0;
	//ball distance
	private double ballDist=0;
	// predicted ball position
	private double predX=0;
	private double predY=0;
	// distance to predicted ball position
	private double distToPred=0;
	//flag whether ball is within penalty area or not
	private boolean ballInPenalty=false;
	
	/**
	 * @param wm	das Weltmodell des Torwarts (Typ: <code>WorldModel</code>)
	 * @param com	der Kommunikationskannal zum Server (Typ: <code>CommunicationChannel</code>)
	 * @see WorldModel
	 * @see CommunicationChannel
	 */
	public Goalie(WorldModel wm, CommunicationChannel com) {
		this.wm = wm;
		communicationChannel = com;
		actor = new Actor(com);
		actions = new Actions(wm,actor);
		
	}
	/**
	 * legt die aktuelle Strategie fest, berechnet dann eine gnstige Aktion und sendet diese an den Server 
	 * @param wm_update ein <code>boolean</code>-Wert, der aussagt, ob die vom Weltmodell bereitsgestellten Daten
	 *		  aktuell sind oder nicht (da "see"-Nachrichten	alle 150 ms, "sense_body" schon alle 100ms)
	 *	<br> <br>Strategiefindung: <br>
	 *	<ul>
	 *	<li> welcher Spielmodus liegt vor?
	 *	<li> berechne taktische Position
	 *	<li> wenn Spielmodus="play_on":
	 *		<ul>
	 *		<li> Wenn keine Strategie aus vorherigen Cycles vorhanden => berechne Strategie
	 *		<li> ansonsten bernehme Strategie aus vorherigen Cycles
	 *		</ul>
	 *	<li> wenn Spielmodus="goalkick_"+eigene Hlfte => fhre Goalkick aus
	 *  </ul>
	 */			
	public void act(boolean wm_update) {
		
		this.wm_update=wm_update;
		// if see-data up-to-date refresh data in Actions-class, else predict data

		if(wm_update) {
			actions.refreshData();
			ballX = wm.getBallX();
			ballY = wm.getBallY();
			ballDist= wm.getBallDistance();}
		else {
			ballX = wm.getPredictedBallPos(1)[0];
			ballY = wm.getPredictedBallPos(1)[1];
			ballDist = Math.sqrt((ballX-wm.getPlayerPos()[0])*(ballX-wm.getPlayerPos()[0])+
								  (ballY-wm.getPlayerPos()[1])*(ballY-wm.getPlayerPos()[1]));
		}
		//set variables
		predX=wm.getPredictedBallPos(3)[0];
		predY=wm.getPredictedBallPos(3)[1];
		distToPred=Math.sqrt((wm.getPlayerPos()[0]-predX)*(wm.getPlayerPos()[0]-predX)+
							 (wm.getPlayerPos()[1]-predY)*(wm.getPlayerPos()[1]-predY));
		if((ballX<-35.5) && (Math.abs(ballY)<16))
			ballInPenalty=true;
		else
			ballInPenalty=false;
				
		
		status = -1;
		// check playmode
		if(wm.getPlayMode().equals("before_kick_off")) 					status = 0;
		if(wm.getPlayMode().startsWith("kick_off")) {
			if(wm.getPlayMode().equals("kick_off_"+wm.getSide())) 		status = 1;
			else														status = 1;
		}
		if(wm.getPlayMode().equals("play_on"))		   					status = 2;
		if(wm.getPlayMode().startsWith("kick_in")) {
			if(wm.getPlayMode().equals("kick_in_"+wm.getSide()))		status = 4;
			else														status = 4;
		}
		if(wm.getPlayMode().startsWith("free_kick")) {
			if(wm.getPlayMode().equals("free_kick_"+wm.getSide()))	 	status = 6;
			else														status = 4;
		}													
		if(wm.getPlayMode().startsWith("corner_kick")) {
			if(wm.getPlayMode().equals("corner_kick_"+wm.getSide())) 	status = 4;
			else														status = 4;
		}
		
		if(wm.getPlayMode().startsWith("goal"))							status = 0;
		
		if(wm.getPlayMode().startsWith("goal_kick")) {
			if(wm.getPlayMode().equals("goal_kick_"+wm.getSide())) 		status = 3;
			else														status = 4;
		}
		
		

		if(wm.getPlayMode().startsWith("offside")) {
			if(wm.getPlayMode().equals("offside_"+wm.getSide()))		status = 4;
			else 									 					status = 4;
		}
		//set tactical position
		setTacticalXY();
		//select what to do based on the current playmode
		switch(status){
		
		
		//Before kick off
		case 0:		if(Math.abs(wm.getPlayerPos()[0]-wm.getStartX())>1 ||Math.abs(wm.getPlayerPos()[1]-wm.getStartY())>1)
						actor.move(wm.getStartX(),wm.getStartY());
					else
						actions.turn(0,0);
					break;
		//kick off					
		case 1:		if((Math.abs(wm.getPlayerPos()[0]-wm.getStartX())>1) || (Math.abs(wm.getPlayerPos()[1]-wm.getStartY())>1))
						actions.run(wm.getStartX(),wm.getStartY());
					else
						actions.turn(0,0);
					break;
							
		//play on
		case 2:			
				//set strategy
				if(strategy<1) {
					if(wm.ballInSight()) {
						if(ballInPenalty)
							strategy=2;
						else
							strategy=1;
						}
					
					
					if(!wm.ballInSight()) {
								strategy=3;
								
								}
					}
				//act
				play();
				break;
		
		
		//goalkick
		case 3:		if(!wm.ballInSight() && !wm_update) 
						break;
					if(!wm.ballInSight()) 
						actions.turn90(); 
					if(wm.ballInSight()) {
						if(ballDist>0.7)
							actions.run(ballX,ballY);
						else
							actions.goalkick();
						}
					break;
		//all other cases
		case 4:		if(distToTacticalPos>1)
						actions.run(tacticalX,tacticalY);
					else {
						if(wm.ballInSight() && wm_update)
							actions.turn(wm.getBallDirection());
						if(!wm.ballInSight() && wm_update)
							actions.turn(0);
						}
					break;
		
		
		//free-kick 
		case 6:		
						
					if(!wm.ballInSight() && !wm_update) 
						break;
					if(!wm.ballInSight()) 
						actions.turn90(); 
					if(wm.ballInSight() && wm.isClosestPlayerToBall() && ballInPenalty) {
						if(wm.getBallDistance()>0.7 || wm.ballInVisibleDistance())
							actions.run(ballX,ballY);
						else
							actions.goalkick();
					}
					break;
		}	
	
	}
	//let the player act	
	private void play() {
		
		//select strategy
		switch(strategy) {
			case 1:		//cancel strategy?
						if((ballInPenalty) || wm.getLastBallUpdate()>7) 
								{strategy=0;
								break;}
						//run to tactical position		
						if(distToTacticalPos>3) {
							actions.run(tacticalX,tacticalY);}
						else { //turn to ball
							if(wm.ballInSight() && wm_update) {
								actions.turn(ballX,ballY);}
						}
						
						break;
						
			case 2:		//cancel strategy?
						if((!ballInPenalty)|| (wm.getLastBallUpdate()>5)) {
								strategy=0;
								break;}
						//run to ball and catch/kick it
						if(wm.getBallDistance()>1.9) {
							if(wm.getBallSpeed()>0.4) {
								if(distToPred>0.5) {
									if(predX>-52.5)
										actions.run(predX,predY);
									else
										actions.run(-52.5,predY);
									}
								else
									actions.turn(ballX,ballY);
								}
							else
								actions.run(ballX,ballY);
								}
						else	
							{ if(wm.getBallDistance()>0.7)
								actor.Catch(wm.getRelBallDirection());
							  else 
							  	actions.goalkick();
							strategy=0;}
						break;
			
			case 3:		//if data up-to-date turn
						if(wm_update) {
								actions.turn90();
							}
						strategy=0; 
						break;							
			
												
		
			}
		
		
		}
		
		//sets the tactical position
		private void setTacticalXY() {
			tacticalX=-50;
			
			tacticalY=0;	

			distToTacticalPos = Math.sqrt((wm.getPlayerPos()[0]-tacticalX)*(wm.getPlayerPos()[0]-tacticalX)
				+(wm.getPlayerPos()[1]-tacticalY)*(wm.getPlayerPos()[1]-tacticalY));
					}	
					

		 
}
		
		
	
