/** Programmname: RoboClient <br>
 * <br>
 * In dieser Klasse befindet sich die Strategie fr die Offensivspieler. Sie kann grob folgendermaen 
 * zusammengefat werden:
 * <ul>
 * <li> Spieler dreht sich, bis er den Ball sieht
 * <li> Wenn er der nchste Spieler des eigenen Teams zum Ball ist, rennt er hin und kickt ihn 
 *		<ul>
 *		<li> auf das gegnerische Tor, ein Gegner nher als 3m steht und die Distanz zum
 *		     gegnerischen Tor < 30 ist oder kein Pass mglich ist
 *		<li> schwach Richtung gegnerisches Tor, falls der nchste Gegner mindestens 3m entfernt ist und
 *			 das gegnerische Tor mindestens 30m entfernt ist
 *		<li> zu einem Mitspieler, falls die Distanz zum gegnerischen Tor > 30 ist, ein Gegner nher als 3m
 *			 steht und die Daten ber diesen Mitspieler nicht veraltet sind	
 *		</ul>
 * <li> Wenn ein anderer Spieler nher zum Ball steht, rennt er zu einer taktisch gnstigen Position
 * </ul>
 * @author Bernd Stieger/Michael Wagner
 * @version 1.0
 */

public class Offense {

	//communication
	private CommunicationChannel communicationChannel;
	//goalie's world model
	private WorldModel wm;
	// actions
	private Actions actions;
	// basic actor
	private Actor actor;
	//playmode
	private int status=-1;
	/*  strategy=1 => look for ball
		strategy=2 => run to ball & do something with it
		strategy=3 => run to strategic pos
		strategy=4 => recover
		*/
	private int strategy=-1;
	//tactical position
	private double tacticalX=0;
	//distance to tactical position
	private double distToTacticalPos=-1;
	//flag for actuality of see-data
	private boolean wm_update=false;
	//ball postion
	private double ballX=0;
	private double ballY=0;
	//ball distance
	private double ballDist=0;
	//predicted ball position
	private double predX=0;
	private double predY=0;
	// distance to postion above
	private double distToPred=0;
	//flag whether player is closest player to position above
	private boolean closestToPred=false;
	
	/**
	 * @param wm	das Weltmodell des Spielers (Typ: <code>WorldModel</code>)
	 * @param com	der Kommunikationskannal zum Server (Typ: <code>CommunicationChannel</code>)
	 * @see WorldModel
	 * @see CommunicationChannel
	 */
	public Offense(WorldModel wm, CommunicationChannel com) {
		this.wm = wm;
		communicationChannel = com;
		actor = new Actor(com);
		actions = new Actions(wm,actor);
		
	}
	/**
	 * legt die aktuelle Strategie fest, berechnet dann eine gnstige Aktion und sendet diese an den Server 
	 * @param wm_update ein <code>boolean</code>-Wert, der aussagt, ob die vom Weltmodell bereitsgestellten Daten
	 *		  aktuell sind oder nicht (da "see"-Nachrichten	alle 150 ms, "sense_body" schon alle 100ms)
	 *	<br> <br>Strategiefindung: <br>
	 *	<ul>
	 *	<li> welcher Spielmodus liegt vor?
	 *	<li> berechne taktische Position
	 *	<li> wenn Spielmodus="play_on":
	 *		<ul>
	 *		<li> Wenn keine Strategie aus vorherigen Cycles vorhanden => berechne Strategie
	 *		<li> ansonsten bernehme Strategie aus vorherigen Cycles
	 *		</ul>
	 *	<li> 
	 *  </ul>
	 */	
	public void act(boolean wm_update) {
		this.wm_update=wm_update;
		// if see-data up-to-date refresh data in Actions-class, else predict data
		if(wm_update) {
			actions.refreshData();
			ballX = wm.getBallX();
			ballY = wm.getBallY();
			ballDist= wm.getBallDistance();
		}
		else {
			ballX = wm.getPredictedBallPos(1)[0];
			ballY = wm.getPredictedBallPos(1)[1];
			ballDist = Math.sqrt((ballX-wm.getPlayerPos()[0])*(ballX-wm.getPlayerPos()[0])+
								  (ballY-wm.getPlayerPos()[1])*(ballY-wm.getPlayerPos()[1]));
		}
		//set variables
		predX=wm.getPredictedBallPos(4)[0];
		predY=wm.getPredictedBallPos(4)[1];
		distToPred=Math.sqrt((wm.getPlayerPos()[0]-predX)*(wm.getPlayerPos()[0]-predX)+
							 (wm.getPlayerPos()[1]-predY)*(wm.getPlayerPos()[1]-predY));
		closestToPred=wm.isClosestPlayerToPredBall(4);
		// check playmode
		status = -1;
		
		if(wm.getPlayMode().equals("before_kick_off")) 					status = 0;
		if(wm.getPlayMode().startsWith("kick_off")) {
			if(wm.getPlayMode().equals("kick_off_"+wm.getSide())) 		status = 10;
			else														status = 0;
		}
		if(wm.getPlayMode().equals("play_on"))		   					status = 2;
		if(wm.getPlayMode().startsWith("kick_in")) {
			if(wm.getPlayMode().equals("kick_in_"+wm.getSide()))		status = 2;
			else														status = 3;
		}
		if(wm.getPlayMode().startsWith("free_kick")) {
			if(wm.getPlayMode().equals("free_kick_"+wm.getSide()))	 	status = 2;
			else														status = 3;
		}													
		if(wm.getPlayMode().startsWith("corner_kick")) {
			if(wm.getPlayMode().equals("corner_kick_"+wm.getSide())) 	status = 2;
			else														status = 3;
		}
		if(wm.getPlayMode().startsWith("goal")) 						status = 0;
		
		if(wm.getPlayMode().startsWith("goal_kick")) {
			if(wm.getPlayMode().equals("goal_kick_"+wm.getSide())) 		status = 3;
			else														status = 5;
		}

		if(wm.getPlayMode().startsWith("offside")) {
			if(wm.getPlayMode().equals("offside_"+wm.getSide()))		status = 7;
			else 									 					status = 3;
		}
		//set tactical position
		setTacticalX();
		//select playmode
		switch(status){
		
		
		//Before kick off
		case 0:	if(Math.abs(wm.getPlayerPos()[0]-wm.getStartX())>1 ||Math.abs(wm.getPlayerPos()[1]-wm.getStartY())>1)
					actor.move(wm.getStartX(),wm.getStartY());
				else
					actions.turn(0,0);		
						break;
					
		//play on
		case 2:			
				//set strategy		
				if(strategy<1) {
					if(!wm.ballInSight()) {
						if(wm.ballInVisibleDistance()) {
							strategy=2;
						}	
						else {
							strategy=1;}
					}
					if(wm.ballInSight()) {
							if(closestToPred || wm.isClosestPlayerToBall()){
								strategy=2;}
							else {
								if(wm.getStamina()<100)
									strategy=4;
								else	
									strategy=3;}
						}
						
					}
				//act	
				play();
				break;
		
		
		//free kick			   
		case 3:		
		//corner kick
		case 4:			
		
				
		//goalkick
		case 5:		
				
		
	
		
		//offside
		case 7:				
		
		
						
					
		//go to tactical position
		case 11:	if(wm.getStamina()>2000) {
						if(distToTacticalPos>3) 
							actions.run(tacticalX,wm.getStartY());
						else {
							if(wm.ballInSight() && wm_update)
								actions.turn(wm.getBallDirection());
							else if(wm_update)
								actions.turn90();
							}	
						}
					break;
		//own kick off			
		case 10:	if(!wm.ballInSight() && wm_update)
						{actions.turn(0,0); break;}
					if(wm.isClosestPlayerToBall()) {
						if(ballDist>0.7)
							actions.run(wm.getBallX(),wm.getBallY());
						else
							actions.goalkick();
					}
	
	
		}
	}
		
	void play() {
		

		switch(strategy) {
			case 1:		//turn only if see-data up-to-date
						if (wm_update) 
							actions.turn90();
						strategy=0; break;
						
			case 2:		//cancel strategy?
						if((wm.getLastBallUpdate()>5)||(!closestToPred && 
								!wm.isClosestPlayerToBall())) 
							{strategy=0; break;}
						if(ballDist>0.7) {
								if(distToPred>0.5)
									actions.run(predX,predY);
								else
									actions.turn(ballX,ballY);}
						//do something with ball
						else {
							if(wm.getOpGoalDist()>30) {
						  	
						  			if(wm.getClosestOpponent().distance()<=3) {
						  				if(wm.getPassTeammate().absX()<wm.getOffsideX()) {
						  			  		if(wm.getPassTeammate().lastUpdate()<5 && wm.getPassTeammate().distance()>4) 
						  	  	    			actions.kickToPos(wm.getPassTeammate().absX(),wm.getPassTeammate().absY());
						  	  	    		
						  	  	  	  		if(wm.getPassTeammate().lastUpdate()>=5) 	
						  	  	    			actions.dribbleToOpGoal();
						  	  	    	}
						  	  	    	if(wm.getPassTeammate().absX()>=wm.getOffsideX()) 
						  	  	    		actions.goalkick();
						  				}
						  		    if(wm.getClosestOpponent().distance()>3)
						  				actions.dribbleToOpGoal();
							}						 
							if(wm.getOpGoalDist()<=30)
								actions.goalkick();
							strategy=0;
						}
						break;
						
			case 3:		//cancel strategy?
						if((wm.getLastBallUpdate()>10)||wm.ballInVisibleDistance()||wm.isClosestPlayerToBall())
								 {strategy=0; break;}
						//run to tactical position
						if(distToTacticalPos>1)
							actions.run(tacticalX,wm.getStartY());
			
						else {
							if(wm.ballInSight() && wm_update) 
								actions.turn(wm.getBallDirection());
							if(!wm.ballInSight() && wm_update) {
								actions.turn90();}
								strategy=0;
								}	
						break;
			
			case 4:		//recover
						if(wm.getStamina()>1000) 
							strategy=0;	
						break;
			
			}
		
		
		}
		
		//set tactical position
		private void setTacticalX() {
						
						if(ballX<=0) 
							 tacticalX=0;
						else {
							if(ballX<45)
								tacticalX=ballX-3;
							else 
								tacticalX=-30;
							}
						if(status==3)
								tacticalX=0;	
							if(status==4)
								tacticalX=0;	
							if(status==5)
								tacticalX=20;
							
						distToTacticalPos=Math.sqrt((wm.getPlayerPos()[0]-tacticalX)*(wm.getPlayerPos()[0]-tacticalX)+
													(wm.getPlayerPos()[1]-wm.getStartY())*(wm.getPlayerPos()[1]-wm.getStartY()));
					}	
}
		
		
	
