/** Programmname: RoboCup-Kamikaze-Client
* Klasse: PlayerPerception
* <hr>
* Diese Klasse reprsentiert einen gesehenen Spieler im Weltmodell des Spielers
* <hr>
* * Standardwerte sind (falls keine Informationen vom Server verfgbar sind):
* <ul>
* <li> <code>int</code>- und <code>double</code>-Werte: 5000
* <li> <code>teamName</code>-Werte: "%==="
* <li> <code>boolean</code>-Werte: false
* </ul>
* @author Michael Wagner
* @version 1.0 
*/

public class PlayerPerception
{
	//Attribute		
	private double absX = 5000; //recent and new absolute coordinates
	private double absY = 5000;
	private double oldX = 5000;
	private double oldY = 5000;
	private int direction = 5000; // body's and neck's direction
	private int neckDir = 5000;
	private double distance = 5000;
	private double distToBall = 5000;
	private double distToOpGoal = 5000;
	private double distToOwnGoal = 5000;
	private String team = "%===";
	private int number = 5000;
	public boolean inSight = false;
	private int lastUpdate;
	private boolean isGoalie = false;

	//Konstruktor
	public PlayerPerception()
	{
	}
	
	//Operationen		
	/**
	* die absolute X-Koordinate
	* @return einen <code>double</code>-Wert zwischen -52.5 - 52.5
	*/
	public double absX()
	{
		return absX;
	}
	
	/**
	* die absolute Y-Koordinate 
	* @return einen <code>double</code>-Wert zwischen -34 - 34
	*/
	public double absY()
	{
		return absY;
	}
	
	/**
	* absolute Richtung des Spielers
	* @return einen <code>int</code>-Wert
	*/
	public int direction()
	{
		return direction;
	}
	
	/**
	* Blickrichtung
	* @return einen<code>int</code>-Wert
	*/
	public int neckDir()
	{
		return neckDir;
	}
	/**
	* Enfernung vom Spieler
	* @return einen <code>double</code>-Wert
	*/
	public double distance()
	{
		return distance;
	}
	
	
	/**
	* Teamname
	* @return einen <code>String</code>
	*/
	public String teamName()
	{
		return team;
	}
	
	/**
	* Trikotnummer
	* @return einen <code>int</code>-Wert
	*/
	public int uniformNumber()
	{
		return number;
	}
	
	/**
	* Entfernung zum Ball
	* @return einen<code>double</code>-Wert
	*/
	public double distToBall()
	{
		return distToBall;
	}
	
	/**
	* Entfernung zum gegnerischen Tor
	* @return einen <code>double</code>-Wert
	*/
	public double distToOpGoal()
	{
		return distToOpGoal;
	}
	
	/**
	* Entfernung zum eigenen Tor
	* @return einen <code>double</code>-Wert
	*/
	public double distToOwnGoal()
	{
		return distToOwnGoal;
	}
	/** ist Spieler Torwart?
	 * @return einen<code> boolean</code>-Wert
	 */
	public boolean isGoalie() {
		return isGoalie;
	}
	
	/**
	* Zeitpunkt der letzten Aktualisierung
	* @return einen <code>int</code>-Wert
	*/
	public int lastUpdate()
	{
		return lastUpdate;
	}
	
	/**
	* aktualisiert die Daten anhand der Informationen vom Server
	* @param objPer die Information vom Server ber den gesehenen Spieler (Typ: <code>ObjectPerception</code>)
	* @param playerPos die Position des Spielers selbst (Typ: <code>double[]</code>
	* @param absPlayerAngle die absolute Blickrichtung des Spielers selbst (Typ: <code>int</code>)
	* @param ballX die absolute X-Koordinate des Balls (Typ: <code>double</code>)
	* @param ballY die absolute Y-Koordinate des Balls (Typ: <code>double</code>)
	* @see <code>ObjectPerception</code>
	* @see <code>WorldModel</code>
	*/
	public void update(ObjectPerception objPer, double[] playerPos, int absPlayerAngle,double ballX,double ballY) {
		//save old data
		oldX = absX;
		oldY = absY;
		//set teamname
		team = objPer.teamName();
		//set uniformNumber
		number = objPer.uniformNumber();
		//set goalie-flag
		isGoalie=objPer.isGoalie();
		//update absolute player direction
		double deltaX, deltaY;
		if(Math.abs(absPlayerAngle)+Math.abs(objPer.direction())<=180)
			direction = absPlayerAngle + objPer.direction();
		else
		{
			if(absPlayerAngle>0)
				direction = absPlayerAngle+objPer.direction()-360;
			if(absPlayerAngle<0)
				direction = 360+(absPlayerAngle+objPer.direction());
		}
		//update distance
		distance = objPer.distance();
		//update distance to ball
		double c = Math.PI;
		double absPlayerPi = c*(direction/180.0);
		double wertcos = Math.cos(absPlayerPi);
		double wertsin = Math.sin(absPlayerPi);
		deltaX = objPer.distance() * wertcos;
		deltaY = objPer.distance() * wertsin;
		absX = playerPos[0]+deltaX;
		absY = playerPos[1]+deltaY;
		double deltaBallX = Math.abs(absX-ballX);
		double deltaBallY = Math.abs(absY-ballY);
		distToBall=Math.sqrt(deltaBallX*deltaBallX+deltaBallY*deltaBallY);
		//set distance to goals
		distToOpGoal = Math.sqrt((52.5-absX)*(52.5-absX)+absY*absY);
		distToOwnGoal = Math.sqrt((-52.5-absX)*(-52.5-absX)+absY*absY);
		//set time
		lastUpdate = objPer.time();	
	}
}