/** Programmname: RoboCup-Kamikaze-Client <br>
* Diese Klasse meldet die Spieler am Server an. Anschlieend wird eine Schleife gestartet, die
* jeweils immer auf eine neue Server-Nachricht wartet, diese dem Weltmodell uebergibt und
* bei einem body event die jeweilige Strategieklasse startet.
* 
* @author Markus Mllerpods/Andreas Lenz
* @version 1.0 / 4.7.02
* @see CommunicationChannel
* @see WorldModel
* @see Goalie
* @see Defense
* @see Midfield
* @see Offense
*/

public class ServerListener extends Thread
{
	//Objektattribute
	private CommunicationChannel communicationChannel;
	private String teamName, host, playerType;
	private int port;
	private Goalie goalie;
	private Defense defense;
	private Midfield midfield;
	private Offense offense;
	public WorldModel wm;
	private boolean wm_update=false;
	

	//Konstruktor
	/** Erzeugt ein neues Objekt und
	* initialisiert es mit dem Teamnamen,
	* dem Host mit zugehrigem Port
	* und einem Playertype ("goalie", "defense", "midfield", "offense")
	* @param teamName teamName
	* @param host host
	* @param port port
	* @param playerType playerType
	*/
	public ServerListener(String teamName, String host, int port, String playerType)
	{
		this.teamName = teamName;
		this.host = host;
		this.port = port;
		this.playerType = playerType;
 	}
   
   	//Operationen
   	/**
   	*Meldet Spieler beim Server an, und initialisiert ihr Weltmodell
   	*/
   	public void init()
   	{
      	String serverMessage, initMessage1, initMessage2;
    	communicationChannel = new CommunicationChannel(host, port);
        
    	if (playerType.equals("goalie"))
           communicationChannel.send(new String("(init " + teamName + "(version 7.0) (goalie))"));
        else
           communicationChannel.send(new String("(init " + teamName + "(version 7.0))"));
        initMessage1 = communicationChannel.receive();
        if (initMessage1.startsWith("(error)"))
       	{
        	System.out.println(initMessage1);
        	System.exit(0);
        }
        initMessage2 = communicationChannel.receive();
        wm = new WorldModel(teamName,initMessage1, initMessage2);

        if(playerType.equals("goalie"))
         goalie = new Goalie(wm, communicationChannel);
		if(playerType.equals("defense"))
		 defense = new Defense(wm, communicationChannel);
		if(playerType.equals("midfield"))
		 midfield = new Midfield(wm, communicationChannel);
		if(playerType.equals("offense"))
		 offense = new Offense(wm, communicationChannel);
	}
	
	/**
	*eine Schleife, die jeweils auf eine Server-Nachricht wartet, diese dem Weltmodell uebergibt
	* und, falls ein body event vorliegt, die Strategieklassen aufruft
	*/
	public void run()
	{    
    	String serverMessage;
    	while(true)
    	{	
    		//wait for new message from server
       	  	serverMessage = communicationChannel.receive();
		
			// it's a visual ("see") event
            if (serverMessage.startsWith("(see"))
            {
               	VisualEvent event = new VisualEvent(serverMessage);
               	wm.ProcessVisualEvent(event);
               	wm_update=true;


    		}
    
    	 	// it's a body ("sense_body") event
            if (serverMessage.startsWith("(sense_body"))
            {
            	BodyEvent event = new BodyEvent(serverMessage);
            	wm.ProcessBodyEvent(event);
            	if(playerType.equals("goalie")) goalie.act(wm_update);
            	if(playerType.equals("defense")) defense.act(wm_update);
            	if(playerType.equals("midfield")) midfield.act(wm_update);
            	if(playerType.equals("offense")) offense.act(wm_update);
				wm_update=false;
            	 
            }
            // it's an aural ("hear") event
            if (serverMessage.startsWith("(hear"))
            {
                AuralEvent event = new AuralEvent(serverMessage);
                wm.ProcessAuralEvent(event);
            }
		}
	}		
}