import java.util.*;
/** Programmname: RoboCup-Kamikaze-Client <br>
* Klasse: BodyEvent
* <hr>
* bereitet die "see"-Nachrichten vom Server fr die Weiterverarbeitung durch das Welmodell vor.
* <hr>
* @see WorldModel
* @author Michael Wagner
* @version 1.0 
*/


public class VisualEvent 
{
	//the ball
    private ObjectPerception ball;
    //closest flag
    private ObjectPerception closestFlag = new ObjectPerception();
    //closest line
    private ObjectPerception closestLine = new ObjectPerception();
    //list of goals
    private Vector goalList = new Vector();
    //list of players
    private Vector playerList = new Vector();

    //Konstruktor
    /**
	* @param message der Server-<code>String</code> der "see"-Nachricht
	*/
    public VisualEvent(String message)
    {
        // use parantheses as string delimiters
        StringTokenizer wholeMessage = new StringTokenizer(message, "()");
        // extract the time stamp
        // - nextToken returns something like "see 15 "
        // - use substring and trim whitespaces
        // Example:
        // The received String is:
        // (see 0 ((g r) 66.7 34) ((b) 20.3 5))
        // The first Token extracted would be:
        // "see 0 " (the blank at the end is included)
        // substring(4) returns "0 "; trim() removes leading and trailing blanks
        // so it returns "0"
        int time = Integer.parseInt(wholeMessage.nextToken().substring(4).trim());
        // process all seen objects
        while (wholeMessage.hasMoreTokens())
        {
        	String objName=wholeMessage.nextToken();
			// sometimes see messages arrive without any objects following
          	if (!objName.equals(" ") && wholeMessage.hasMoreTokens())
          	{
				//tokenize data-block
				String data = wholeMessage.nextToken();
            	StringTokenizer dataTok = new StringTokenizer(data," ");
	            //set default values
                double distance = 5000;
                int direction = 5000;
                double distChange = 5000;
                double dirChange = 5000;
                int bodyFacingDir = 5000;
                int headFacingDir = 5000;
                String teamName = "%===";
                int uniformNumber = 5000;
                boolean isGoalie=false;
				
				try
				{	//which kind of object?
            		switch (objName.charAt(0))
            		{   //ball
              			case 'B':	//no info available?
              			case 'b':   if(dataTok.countTokens()<1)
              					    {
              					  		break;
              					    }
              					    //only direction available
              						if(dataTok.countTokens()==1)
              						{
              							direction = Integer.parseInt(dataTok.nextToken());
                                  	 	ball = new ObjectPerception(time,objName,distance,direction,distChange,dirChange,
                                                     				bodyFacingDir,headFacingDir,teamName,uniformNumber);
                                    	break;
                                    }
                                    //all infos available
                                 	try
                                 	{
                                        distance = Double.parseDouble(dataTok.nextToken());
                                        direction = Integer.parseInt(dataTok.nextToken());
                                        distChange = Double.parseDouble(dataTok.nextToken());
                                        dirChange = Double.parseDouble(dataTok.nextToken());
                                    }
                                    //element not available
                                 	catch (NoSuchElementException e) {}
                                 	catch (NumberFormatException e2) {}
                                 	//save ball
                                 	ball = new ObjectPerception(time,objName,distance,direction,distChange,dirChange,
                                                       			bodyFacingDir,headFacingDir,teamName,uniformNumber);
                                 	break;
                        //goal 
             			case 'g':   //no infos avilable
             						if(dataTok.countTokens()<2)
             						 	break;
              			  			try
              			  			{	distance = Double.parseDouble(dataTok.nextToken());
                                     	direction = Integer.parseInt(dataTok.nextToken());}
                                 	//element not available?
                                 	catch(NumberFormatException e) {}
                                 	//add goal to list
                                 	goalList.addElement(new ObjectPerception(time,objName,distance,direction,distChange,dirChange,
                                                     		bodyFacingDir,headFacingDir,teamName,uniformNumber));
                                	break;
              			//flag
              			case 'f':   if(dataTok.countTokens()<=1)
              							break;
              						try
                                 	{	distance = Double.parseDouble(dataTok.nextToken());
                                 		direction = Integer.parseInt(dataTok.nextToken());}
                                 	//no infos available
                                 	catch(NumberFormatException e)
                                 		{break;}
                                 	//not closest flag => cancel
                                 	if(closestFlag.distance()<=distance)
                                 		{break;}
                                 	//save flag	
                                 	closestFlag=new ObjectPerception(time,objName,distance,direction,distChange,dirChange,
                                                       				 bodyFacingDir,headFacingDir,teamName,uniformNumber);
                                 	break;
 						//line
             			case 'l':  	//no infos avilable
             						if(dataTok.countTokens()<=1)
             							break;
             						
                                	try
                                	{	distance = Double.parseDouble(dataTok.nextToken());
                                		direction = (int)Double.parseDouble(dataTok.nextToken());
                                		if(closestLine.distance()<=distance)
                                			break;
                                	}
                                	//element not available
                                	catch (NumberFormatException e) {}
                                	//save closest line
                                	closestLine=new ObjectPerception(time,objName,distance,direction,distChange,dirChange,
                                                     				 bodyFacingDir,headFacingDir,teamName,uniformNumber);
                              	    break;                  
              			 //player
           			  	 case 'p': 	//no infos avilable
           			  	 			if(dataTok.countTokens()<1)
           			  	 				break;
           			  	 			//tokenize objectname-part
              						StringTokenizer playerTok = new StringTokenizer(objName," ");
                                	playerTok.nextToken(); 
                                	//skip'p'
                                	//save team,number & goalie-flag
                                	if(playerTok.hasMoreElements())
                                	{   teamName = playerTok.nextToken();
                                        teamName = teamName.substring(1,teamName.length()-1);}
                                    if(playerTok.hasMoreElements())
                                		uniformNumber = Integer.parseInt(playerTok.nextToken());
                                	if(playerTok.hasMoreElements())
                                		isGoalie=true;
                                	//only direction available	
                                	if(dataTok.countTokens()==1)
                                	{	direction = (int)Double.parseDouble(dataTok.nextToken());
                                        playerList.addElement(new ObjectPerception(time,distance,direction,distChange,dirChange,
                                            									   bodyFacingDir,headFacingDir,teamName,uniformNumber,isGoalie));
                                       	break;}
                                       	
                              		 //all infos available
                              		 try
                              		 {  distance = Double.parseDouble(dataTok.nextToken());
                                        direction = Integer.parseInt(dataTok.nextToken());
                                        distChange = Double.parseDouble(dataTok.nextToken());
                                        dirChange = Double.parseDouble(dataTok.nextToken());
                                        bodyFacingDir = Integer.parseInt(dataTok.nextToken());
                                        headFacingDir = Integer.parseInt(dataTok.nextToken());
                                     }
                               		 //element not available
                               		 catch(NoSuchElementException e) {}
                               		 catch(NumberFormatException e2) {}
                               		 //add player to list
                                     playerList.addElement(new ObjectPerception(time,distance,direction,distChange,dirChange,
                                                       						    bodyFacingDir,headFacingDir,teamName,uniformNumber));
                              		 break;                          
               
           				}
       				}
       				//catch strange errors that sometimes appear
         			catch (NumberFormatException e2) {}
       	 			catch (NoSuchElementException e3) {}
         		 }
       		 }
    }
 
	//Operationen
    
    /**
	* die Server-Informationen ber den Ball
	* @return ein <code>ObjectPerception</code> des Balls
	* @see <code>ObjectPerception</code>
	*/
    public ObjectPerception ball()
    {
    	return ball;
    }
    
    /**
	* die Server-Informationen ber die nchste Fahne
	* @return ein <code>ObjectPerception</code> der Fahne
	* @see <code>ObjectPerception</code>
	*/    
    public ObjectPerception closestFlag()
    {
    	return closestFlag;
    }
    
    /**
	* die Server-Informationen ber die nchste Linie
	* @return ein <code>ObjectPerception</code> der Linie
	* @see <code>ObjectPerception</code>
	*/   
    public ObjectPerception closestLine()
    {
    	return closestLine;
    }
    
    /**
	* die Server-Informationen ber die Tore
	* @return einen <code>Vector</code>, der die <code>ObjectPerceptions</code> der die Tore enthlt
	* @see <code>ObjectPerception</code>
	*/     
    public Vector goalList()
    {
    	return goalList;
    }
        
    /**
	* die Server-Informationen ber die gesehenen Spieler
	* @return einen <code>Vector</code>, der die <code>ObjectPerceptions</code> der die Spieler enthlt
	* @see <code>ObjectPerception</code>
	*/       
    public Vector playerList()
    {
    	return playerList;
    }   
}
