import java.util.*;

/** Programmname: RoboCup-Kamikaze-Client 
* Klasse: BodyEvent
* <hr>
*  Diese Klasse generiert und verwaltet das Weltmodell des Spielers 
* <hr>
* <br>
* Funktionsweise: <br>
* <ul>
* <li> ber die Methoden <code>ProcessVisualEvent, ProcessBodyEvent, ProcessAuralEvent</code> werden
*	   die von den Klassen <code>VisualEvent, BodyEvent, AuralEvent</code> vorverarbeiteten Daten
*	   ins Weltmodell geladen
* <li> aus den Informationen ber die nchste Linie und die nchste	Fahne wird der absolute Blickwinkel
*	   des Spielers sowie die absolute Position auf dem Spielfeld berechnet	
* <li> Die Klassen <code>BallPerception, PlayerPerception, GoalPerception</code> berechnen aus den
*	   vom Server bermittelten Daten fr den Spieler relevante Informationen. Informationen ber
*	   gesehene Spieler werden auerdem in einer Art Gedchtnis des Spielers gespeichert
* <li> Falls ein Objekt in der aktuellen Servernachricht nicht enthalten ist (z.B. Ball), werden die
*	   alten Daten beibehalten  
* </ul>
* @see PlayerPerception		 	
* @see BallPerception
* @see GoalPerception
* @author Michael Wagner/Kamil
* @version 1.0 
*/
  public class WorldModel 
  
    {	
    	//basic memory
    	//--------------
    	private String teamName = "";
		//body direction
		private int absPlayerAngle = 0; 
		//view direction
		private int headAngle = 0; 
		private double velocity = 0;
		private double stamina = 0;
		private String viewWidth="";
		private String viewQuality="";
        //player's side
        private String side = "";
        //player's uniform number
        private int unum = 0;
        private String playMode = "";
        private int time = -1;
        private int[] score = {0,0};
        //-------------------------
        //player's position
        private double playerPos[] = {5000,5000};
        //infos about ball
        private BallPerception ball = new BallPerception();
        //infos about goals
        private GoalPerception ownGoal = new GoalPerception(true);
        private GoalPerception opGoal = new GoalPerception(false);
        //memory for teammates and opponents
        private PlayerPerception[] ownTeam = new PlayerPerception[12];
        private PlayerPerception[] opponent= new PlayerPerception[12];
        //list of currently seen players
        private Vector seenPlayers = new Vector();
        //flags
        private boolean closestPlayerToBall = true;
        private boolean closestPlayerToOpGoal = true;
        //info about closest teammate to opponent goal
        private PlayerPerception closestTeammateToOpGoal = new PlayerPerception();
        //info about a pass partner
        private PlayerPerception possiblePassTeammate = new PlayerPerception();
        //info about closest opponent
        private PlayerPerception closestOpponent = new PlayerPerception();
        //the current offside line
        private double offsideX = 0;   
    

      //----------------------constructor-------------------------------
      /**
       * @param teamname der Teamname des Spielers (Typ:<code>String</code>)
       * @param initMessage1 die erste Nachricht des Servers nach der Anmeldung (Typ:<code>String</code>)
       * @param initMessage2 die zweite Nachricht des Servers nach der Anmeldung (Typ:<code>String</code>)
       */
      public WorldModel(String teamName, String initMessage1, String initMessage2) {
      	//set teamname
      	this.teamName = teamName;
		//Tokenize server messages
      	StringTokenizer msg1 = new StringTokenizer(initMessage1,"() ");
        StringTokenizer msg2 = new StringTokenizer(initMessage2, "() ");
        //skip "init"
        msg1.nextToken();
        //set side, uniform number & playMode
        side = msg1.nextToken();
        unum = Integer.valueOf(msg1.nextToken()).intValue();
      	playMode = msg1.nextToken();	
		//init memory for players
      	for(int i=1;i<11;i++) {
      		opponent[i] = new PlayerPerception();
      	}
      	for(int i=1;i<11;i++) {
      		ownTeam[i] = new PlayerPerception();
      	}
      	
      }
 		/**
 		 * verarbeitet eine "see"-Nachricht
 		 *
 		 * @param VisualEvent die von der Klasse VisualEvent verarbeitete Nachricht
 		 * @see VisualEvent
 		 */
       public void ProcessVisualEvent(VisualEvent event) {
  			VisualEvent ve = event;
  			//clear players-list
  			seenPlayers.clear();
      		
      		//reset flags
      		ball.inSight = false;
      		ownGoal.inSight = false;
      		opGoal.inSight = false;
      		ball.inVisibleDistance=false;;
      		for(int i=1;i<11;i++) {ownTeam[i].inSight=false;
      							   opponent[i].inSight=false;}

			//update ball-flags
			if(ve.ball()!=null) {
      			if(ve.ball().objName().equals("b")) ball.inSight=true; 
      			if(ve.ball().objName().charAt(0)=='B') ball.inVisibleDistance=true;
      													
      		}     		
			//update goal-flags
			ObjectPerception objPerGoal=new ObjectPerception();
      		for(Enumeration e=ve.goalList().elements();e.hasMoreElements() ;) {
      			objPerGoal=(ObjectPerception)e.nextElement();
      			if(objPerGoal.objName().startsWith("g "+side)) 
      				ownGoal.inSight=true;
      			else
      				opGoal.inSight=true;
      		}
      		
             	
  
			//update player's view direction
  			absPlayerAngle = absPlayerAngle(ve.closestLine(), side);
  			//update player's position			
  			playerPos = ownPosition(ve.closestFlag(), ve.closestLine());
  			//update ball
  			if((ball.inSight)||(ball.inVisibleDistance)) 
  				ball.update(ve.ball(),playerPos,absPlayerAngle,time);
  			//update goals						
  			ownGoal.update(playerPos);
  			opGoal.update(playerPos);

        	//create a list of seen players and calculate their position, etc
        	for(Enumeration e=ve.playerList().elements();e.hasMoreElements() ;) {
        		ObjectPerception objPerPlayer=(ObjectPerception)e.nextElement();
        		PlayerPerception playPer = new PlayerPerception();
        		playPer.update(objPerPlayer,playerPos,absPlayerAngle,ball.absX(),ball.absY());
	       		seenPlayers.addElement(playPer);
 
        	}
        
        	//process seen player's list
        	double teammateBallDist=5000;
        	double teammateGoalDist=5000;
        	double minOpponentToGoalDist=5000;
        	double teammatePlayerDist=5000;
        	double minOpponentDist=5000;
        	
        	for(Enumeration e=seenPlayers.elements();e.hasMoreElements() ;) {
        		PlayerPerception playPer = (PlayerPerception)e.nextElement();
        		//own Player?
        		if(playPer.teamName().equals(teamName)) {
        			//uniform number available?
        			if(!(playPer.uniformNumber()==5000)) 
        				//save in memory
        				ownTeam[playPer.uniformNumber()]=playPer;
        			//set teammate's minimum ball distance
        			if(playPer.distToBall()<teammateBallDist) 
        				teammateBallDist=playPer.distToBall();
        			//set teammate's minimum goal distance & save teammate	
        			if(playPer.distToOpGoal()<teammateGoalDist) {
        				teammateGoalDist=playPer.distToOpGoal();	
        				closestTeammateToOpGoal=playPer;}
        			//set "teammate - player" distance and save closest teammate to player
        			if(playPer.distance()<teammatePlayerDist && playPer.absX()>playerPos[0]) {
        				teammatePlayerDist=playPer.distance();
        				possiblePassTeammate=playPer;}
        		}
        		//it's an opponent player
        		if(!playPer.teamName().equals(teamName) && (!playPer.teamName().equals("%==="))) {
        			//save player if number available
        			if(!(playPer.uniformNumber()==5000)) 
        				opponent[playPer.uniformNumber()]=playPer;
        			//set offside line
        			if(52.5-playPer.absX()<minOpponentToGoalDist) 
        				minOpponentToGoalDist = Math.abs(playPer.absX());
        			//save closest opponent to player
        			if((playPer.distance()<minOpponentDist) && (!playPer.isGoalie()))
        				closestOpponent=playPer;
        		}
        		
        	}
        	//update "player"-flags
        	if(teammateBallDist<ball.distance()) 
        		closestPlayerToBall=false;
        	else
        		closestPlayerToBall=true;
        		
        	if(teammateGoalDist<opGoal.distance()) 
        		closestPlayerToOpGoal=false;
        	else
        		closestPlayerToOpGoal=true;
        	//set offside line
        	if(minOpponentToGoalDist<50)
        		offsideX = minOpponentToGoalDist;

       }
        
       /* die absolute Blickrichtung
        * @param line	die <code>ObjectPerception</code> einer Linie
        * @param side	die Spielseite (Typ: <code>String</code>)
        * @return ein <code>int</code>-Wert zwischen -180 und 180 
        * @return die Winkel, wenn der Spieler sich am Anstosspunkt befindet und von links nach rechts spielt:
        * @return - die Richtung des gegnerischen Tors ist die Null-Linie
        * @return - das eigene Tor befindet sich bei -180	
        * @return - die obere Line befindet sich bei -90
        * @return - die untere Line befindet sich bei +90
        */			
       private int absPlayerAngle(ObjectPerception line, String side) {
        	 int absWinkel = 360;
        	 
        	 //line is available?
        	 try {
			 	//relative direction to line
			 	int relWinkel = line.direction();
			 	       	
        	 	//left side?
        	 	if(side.equals("l")) {             
        	       	//which line?
        	       	switch (line.objName().charAt(2)) {
                 		case 't':
                      		//calculate absolute direction
                      		if (relWinkel < 0)
                        		absWinkel = -180 - relWinkel;
                      		if (relWinkel > 0)
                        		absWinkel = (0 - relWinkel);
                      		break;
                 		case 'b':
                      		if (relWinkel < 0)
                        		absWinkel = (0 - relWinkel);
                      		if (relWinkel > 0)
                        		absWinkel = (180 - relWinkel);
                      		break;
                 		case 'l':
                      		if (relWinkel < 0)
                        		absWinkel = (90 - relWinkel);
                      		if (relWinkel > 0)
                        		absWinkel = -(90 + relWinkel);
                      		break;
                 		case 'r':
                      		if (relWinkel < 0)
                        		absWinkel = -(90 + relWinkel);
                      		if (relWinkel > 0)
                        		absWinkel = (90 - relWinkel);
                      		break;
                   }
               }
               //right side?
               if(!side.equals("l")) {
                 	switch (line.objName().charAt(2)) {
               			case 't':
                      		if (relWinkel < 0)
                        		absWinkel = 0- relWinkel;
                      		if (relWinkel > 0)
                        		absWinkel = (180 - relWinkel);
                      		break;
                 		case 'b':
                      		if (relWinkel < 0)
                        		absWinkel = (-180 - relWinkel);
                      		if (relWinkel > 0)
                        		absWinkel = (0 - relWinkel);
                      		break;
                 		case 'l':
                      		if (relWinkel < 0)
                        		absWinkel = -(90 + relWinkel);
                      		if (relWinkel > 0)
                        		absWinkel = (90 - relWinkel);
                      		break;
                 		case 'r':
                      		if (relWinkel < 0)
                        		absWinkel = 90 - relWinkel;
                      		if (relWinkel > 0)
                        		absWinkel = (-90 + relWinkel);
                      		break;
                   }
               }  
             }
             //no line available
             catch(StringIndexOutOfBoundsException e) 
             //return old direction
			 	{absWinkel=absPlayerAngle;}
			//return absolute direction
			return absWinkel;
		}
      	
        /* die absolute Spielerposition
         * @param	flag	eine <code>ObjectPerception</code> einer Fahne
         * @param	line	eine <code>ObjectPerception</code> einer Linie
         * @return ein <code>double[2]</code>-Wert des Formats (x,y)
         * @return dabei gilt: 
         * @return - Spieler in eigener Hlfte: x negativ
         * @return - Spieler in gegnerischer Hlfte: x positiv
         * @return - Spieler unterhalb der Tore: y positiv
         * @return - Spieler oberhalb der Tore: y negativ
         */			
        private double[] ownPosition(ObjectPerception flag,ObjectPerception line) {
        
          int relWinkel = 0;        //relative Winkel bezueglich der Linie
          int absWinkelFlag = 0;  //absoluter Winkel, in dem die Fahne steht	
          double deltaX = 0;      //auf die Fahne zeigender Vektor (X Komponente)
          double deltaY = 0;      //auf die Fahne zeigender Vektor (Y Komponente)
		  double[] a = {5000,5000};  //dummy, der ergebnis speichert
		  int c_absPlayerAngle=0;	//absoluter Winkel bzgl linker Spielseite
          
          relWinkel = line.direction();
          int absWinkel=0;
		  //calculate absolute direction for left side
		  try {          
 				
         	//which line?
               	switch (line.objName().charAt(2)) {
                 		case 't':
                      		//calculate absolute direction
                      		if (relWinkel < 0)
                        		absWinkel = -180 - relWinkel;
                      		if (relWinkel > 0)
                        		absWinkel = (0 - relWinkel);
                      		break;
                 		case 'b':
                      		if (relWinkel < 0)
                        		absWinkel = (0 - relWinkel);
                      		if (relWinkel > 0)
                        		absWinkel = (180 - relWinkel);
                      		break;
                 		case 'l':
                      		if (relWinkel < 0)
                        		absWinkel = (90 - relWinkel);
                      		if (relWinkel > 0)
                        		absWinkel = -(90 + relWinkel);
                      		break;
                 		case 'r':
                      		if (relWinkel < 0)
                        		absWinkel = -(90 + relWinkel);
                      		if (relWinkel > 0)
                        		absWinkel = (90 - relWinkel);
                      		break;
                   }
           }
           catch(StringIndexOutOfBoundsException e) 
           	{if(side.equals("l")) 
           		absWinkel=absPlayerAngle;
           	 else {
           	 	if(absPlayerAngle-180>=-180) 
           	 		absWinkel=absPlayerAngle-180;
           	 	else
           	 		absWinkel=180+absPlayerAngle;
           	 	}
           	 }
          c_absPlayerAngle=absWinkel;
          //Der absolute Winkel in dem die Flagge steht
          absWinkelFlag = c_absPlayerAngle + flag.direction();
          //Bestimmung eines Vektors im zweidimensionalen euklidischen Koordinatensystem
          double c = Math.PI;
          double absWinkelFlagPi = c*(absWinkelFlag/180.0);
          double wertcos = Math.cos(absWinkelFlagPi);
          double wertsin = Math.sin(absWinkelFlagPi);
          deltaX = flag.distance() * wertcos;
          deltaY = flag.distance() * wertsin;
		  
		  try {	
          //Bestimmung eigener Position  a[0] = x, a[1] = y
          if (flag.objName().equals("f c"))  {
  	      	a[0] = 0 - deltaX;
            a[1] = 0 - deltaY;}
          
          if (flag.objName().equals("f c t"))  {
            a[0] = 0 - deltaX;
            a[1] = (-34) - deltaY;}
                                   
          if (flag.objName().equals("f c b"))  {
            a[0] = 0 - deltaX;
            a[1] = 34 - deltaY;}
          
          if (flag.objName().equals("f l t"))  {
            a[0] = (-52.5) - deltaX;
            a[1] = (-34) - deltaY;}
          
          if (flag.objName().equals("f g l t"))  {
            a[0] = (-52.5) - deltaX;
            a[1] = (-7.01) - deltaY;}
                                                            
          if (flag.objName().equals("f g l b"))  {
            a[0] = (-52.5) - deltaX;
            a[1] = 7.01 - deltaY;}
                                                            
          if (flag.objName().equals("f l b"))  {
            a[0] = (-52.5) - deltaX;
            a[1] = 34 - deltaY;}
                                                            
          if (flag.objName().equals("f p l t"))  {
            a[0] = (-36) - deltaX;
                     a[1] = (-20.18) - deltaY;}
                                                     
          if (flag.objName().equals("f p l c"))  {
            a[0] = (-36) - deltaX;
            a[1] = 0 - deltaY;
                                                            }
          if (flag.objName().equals("f p l b"))  {
            a[0] = (-36) - deltaX;
            a[1] = 20.18 - deltaY;
                                                            }
          if ( flag.objName().equals("f r t"))  {
            a[0] = 52.5 - deltaX;
            a[1] = (-34) - deltaY;
                                                            }
          if ( flag.objName().equals("f g r t"))  {
            a[0] = 52.5 - deltaX;
            a[1] = (-7.01) - deltaY;
                                                            }
          if ( flag.objName().equals("f g r b"))  {
            a[0] = 52.5 - deltaX;
            a[1] = 7.01 - deltaY;
                                                            }
          if ( flag.objName().equals("f r b"))  {
            a[0] = 52.5 - deltaX;
            a[1] = 34 - deltaY;
                                                            }
          if ( flag.objName().equals("f p r t"))  {
            a[0] = 36 - deltaX;
            a[1] = (-20.18) - deltaY;
                                                            }
          if ( flag.objName().equals("f p r c"))  {
            a[0] = 36 - deltaX;
            a[1] = 0 - deltaY;
                                                            }
          if ( flag.objName().equals("f p r b"))  {
            a[0] = 36 - deltaX;
            a[1] = 20.18 - deltaY;
                                                            }


          if ( flag.objName().equals("f t l 50"))  {
            a[0] = (-50) - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t l 40"))  {
            a[0] = (-40) - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t l 30"))  {
            a[0] = (-30) - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t l 20"))  {
            a[0] = (-20) - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t l 10"))  {
            a[0] = (-10) - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t 0"))  {
            a[0] = 0 - deltaX;
            a[1] = (-39) - deltaY;
                                                                 }
          if ( flag.objName().equals("f t r 10"))  {
            a[0] = 10 - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t r 20"))  {
            a[0] = 20 - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t r 30"))  {
            a[0] = 30 - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t r 40"))  {
            a[0] = 40 - deltaX;
            a[1] = (-39) - deltaY;
                                                            }
          if ( flag.objName().equals("f t r 50"))  {
            a[0] = 50 - deltaX;
            a[1] = (-39) - deltaY;
                                                            }


          if ( flag.objName().equals("f b l 50"))  {
            a[0] = (-50) - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b l 40"))  {
            a[0] = (-40) - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b l 30"))  {
            a[0] = (-30) - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b l 20"))  {
            a[0] = (-20) - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b l 10"))  {
            a[0] = (-10) - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b 0"))  {
            a[0] = 0 - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b r 10"))  {
            a[0] = 10 - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b r 20"))  {
            a[0] = 20 - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b r 30"))  {
            a[0] = 30 - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b r 40"))  {
            a[0] = 40 - deltaX;
            a[1] = 39 - deltaY;
                                                            }
          if ( flag.objName().equals("f b r 50"))  {
            a[0] = 50 - deltaX;
            a[1] = 39 - deltaY;
                                                            }

          if ( flag.objName().equals("f l t 30"))  {
            a[0] = (-57.5) - deltaX;
            a[1] = (-30) - deltaY;
                                                            }
          if ( flag.objName().equals("f l t 20"))  {
            a[0] = (-57.5) - deltaX;
            a[1] = (-20) - deltaY;
                                                            }
          if ( flag.objName().equals("f l t 10"))  {
            a[0] = (-57.5) - deltaX;
            a[1] = (-10) - deltaY;
                                                            }
          if ( flag.objName().equals("f l 0"))  {
            a[0] = (-57.5) - deltaX;
            a[1] = 0 - deltaY;
                                                            }
          if ( flag.objName().equals("f l b 10"))  {
            a[0] = (-57.5) - deltaX;
            a[1] = 10 - deltaY;
                                                            }
          if ( flag.objName().equals("f l b 20"))  {
            a[0] = (-57.5) - deltaX;
            a[1] = 20 - deltaY;
                                                            }
          if ( flag.objName().equals("f l b 30"))  {
            a[0] = (-57.5) - deltaX;
            a[1] = 30 - deltaY;
                                                            }

          if ( flag.objName().equals("f r t 30"))  {
            a[0] = 57.5 - deltaX;
            a[1] = (-30) - deltaY;
                                                            }
          if ( flag.objName().equals("f r t 20"))  {
            a[0] = 57.5 - deltaX;
            a[1] = (-20) - deltaY;
                                                            }
          if ( flag.objName().equals("f r t 10"))  {
            a[0] = 57.5 - deltaX;
            a[1] = (-10) - deltaY;
                                                           }
          if ( flag.objName().equals("f r 0"))  {
            a[0] = 57.5 - deltaX;
            a[1] = 0 - deltaY;
                                                            }
          if ( flag.objName().equals("f r b 10"))  {
            a[0] = 57.5 - deltaX;
            a[1] = 10 - deltaY;
                                                            }
          if ( flag.objName().equals("f r b 20"))  {
            a[0] = 57.5 - deltaX;
            a[1] = 20 - deltaY;
                                                            }
          if ( flag.objName().equals("f r b 30"))  {
            a[0] = 57.5 - deltaX;
            a[1] = 30 - deltaY;
                                                            }
        }
        //no flag available
       	catch(StringIndexOutOfBoundsException e) 
        	 {return playerPos;}
        //right side=> turn positons around   
        if(!side.equals("l")) {a[0] *= (-1);
          						  a[1] *= (-1);}

        return a;
   		
    }
        	
       /**
        * verarbeitet eine "sense_body"-Nachricht des Servers
        *
        * @param BodyEvent die von der Klasse BodyEvent verarbeitete Nachricht
 		* @see BodyEvent
        */ 	
       public void ProcessBodyEvent(BodyEvent event) {
       		BodyEvent be = event;
       		stamina = be.stamina();
       		time = be.time();
       		velocity = be.speed();
       		headAngle = be.headAngle();
       		viewWidth= be.viewWidth();
       		viewQuality= be.viewQuality();
       }
       
       /**
        * verarbeitet eine "hear"-Nachricht des Servers
        *
        * @param AuralEvent die von der Klasse AuralEvent verarbeitete Nachricht
 		* @see AuralEvent
        */ 	
       public void ProcessAuralEvent(AuralEvent event) {
       		for(Enumeration e=(event.elements());e.hasMoreElements();){
              ObjectPerceptionAural hearMsg = (ObjectPerceptionAural)e.nextElement();
              //update PlayMode
              if (hearMsg.sender().equals("referee")) {
              	playMode = hearMsg.message();
            	//update Score
            	if(hearMsg.message().startsWith("goal")) {
            			if(hearMsg.message().startsWith("goal_"+side)) 
            				score[0]++;
	           			else if(!hearMsg.message().startsWith("goal_k") && !hearMsg.message().startsWith("goalie_c")){ 
            				score[1]++;}
              			}
            	}
              }
        	
       }
	
      
      
      //----------------------------------------Output-Methods------------------------//
      
      
      
      /**
       * sieht der Spieler den Ball?
       *
       * @return ein <code>boolean</code>-Wert
       * @return true => Spieler sieht Ball, false sonst
       */
      public boolean ballInSight() {return ball.inSight;}
      
      /**
       * ist Ball innerhalb der "visible_distance"? 
       * 
       * @return ein <code>boolean</code> 
       * @return true, wenn sich der Spieler innerhalb der "visible_distance" befindet, ansonsten false
       * @return Beachte: kann nur der Fall sein, wenn Spieler den Ball nicht sieht
       */
      public boolean ballInVisibleDistance() {return ball.inVisibleDistance;}
      
      							
      /**						
       * die absolute Richtung des Balls
       *
       * @return ein <code>int</code> Wert zwischen -180 und 180
       * @return Beachte: wenn der Spieler den Ball nicht sieht, steht hier die Richtung,
       * 	     in der der Spieler den Ball zuletzt gesehen hat
       */
      
      public int getBallDirection() {return ball.direction();}
      
      /**
       * Entfernung zum Ball
       *
       * @return ein <code>int</code>-Wert zwischen 0 und 2.7
       * @return Beachte: wenn der Spieler den Ball nicht sieht, steht hier die Entferung,
       * 	     in der der Spieler den Ball zuletzt gesehen hat
       * 
       */
      public double getBallDistance() {return ball.distance();}
      
      /**
       * Geschwindigkeit des Balls
       *
       * @return ein <code>int</code>-Wert
       * @return Beachte: wenn der Spieler den Ball nicht sieht, steht hier die Geschwindigkeit,
       * 	     mit der der Spieler den Ball zuletzt gesehen hat
       */
       public double getBallSpeed() {return ball.speed();}
      /**
       * die X-Koordinate des Balls
       *
       * @return ein <code>double</code>-Wert zwischen -52.5 und 52.5
       * @return Beachte: wenn der Spieler den Ball nicht sieht, steht hier die Position,
       * 	     an der der Spieler den Ball zuletzt gesehen hat
       */
      public double getBallX() {return ball.absX();}
      
       /**
       * die Y-Koordinate des Balls
       *
       * @return ein <code>double</code>-Wert zwischen -34 und 34
       * @return Beachte: wenn der Spieler den Ball nicht sieht, steht hier die Position,
       * 	     an der der Spieler den Ball zuletzt gesehen hat
       */
      public double getBallY() {return ball.absY();}
      							
      /**
       * der absolute Winkel des Krpers
       * 
       * @return ein <code>int</code>-Wert zwischen -180 und 180
       */
       
      public int getBodyDirection() {if(Math.abs(absPlayerAngle+headAngle)<=180) 
      									return (absPlayerAngle+headAngle);
      								else {
      									if(absPlayerAngle>0)
      										return(absPlayerAngle+headAngle-360);
      									else
      										return(360+(absPlayerAngle+headAngle));
									}
      								}
      
   	  /**
       * der gegnerische Spieler, der am nchsten zum Spieler steht
       * @return ein Objekt vom Typ <code>PlayerPerception</code>
       * @see PlayerPerception
       */
   	   
   	  public PlayerPerception getClosestOpponent() {return closestOpponent;}

      /**
       * der Winkel des Kopfes relativ zum Krper
       * 
       * @return ein <code>int</code>-Wert zwischen -90 und 90
       */
      public int getHeadAngle() {return headAngle;}
      /**
       * die Abseitslinie (X-Koordinate des gegnerischen Spielers, der am nchsten zum Tor steht
       *
       * @return ein <code>double</code>-Wert
       * @return Falls der Spieler noch keinen gegnerischen Spieler gesehen hat, ist dieser Wert 5000
       */
      public double getOffsideX() {return offsideX;}
       
      /**
       * Entfernung zum gegnerischen Tor; Dieser Wert ist - unabhngig, davon, ob der Spieler das Tor sieht - immer aktuell
       *
       * @return ein <code>double</code>-Wert
       */
      public double getOpGoalDist() {return opGoal.distance();}
      
      /**
       * Richtung des eigenen Tors; Dieser Wert ist - unabhngig, davon, ob der Spieler das Tor sieht - immer aktuell
       *
       * @return ein <code>int</code>-Wert
       */
      public int getOwnGoalDirection() {return ownGoal.direction();}
      
      /**
       * Richtung des eigenen Tors; Dieser Wert ist - unabhngig, davon, ob der Spieler das Tor sieht - immer aktuell
       *
       * @return ein <code>int</code>-Wert
       */
      public int getOpGoalDirection() {return opGoal.direction();}
      /**
       * Entfernung zum eigenen Tor; Dieser Wert ist - unabhngig, davon, ob der Spieler das Tor sieht - immer aktuell
       *
       * @return ein <code>double</code>-Wert
       */
      public double getOwnGoalDist() {return ownGoal.distance();}
      /**
        * absolute Position des Spielers
        *
        * @return ein <code>double[]</code>-Wert im Format [x,y]
        * @return mgliche Werte: -52.5<x<52.5 und -34<y<34
        */
      public double[] getPlayerPos() {return playerPos;}
        
      /**
       * aktueller Spielmodus
       *
       * @return einen <code>String</code>; 
       * @return mgliche Werte: siehe RCSS-Server Manual S. 45/46
       */
      public String getPlayMode() {return playMode;}
      
      
      /**
       * wieviel Zeit vergangen ist, seit der Spieler den Ball zuletzt gesehen hat
       *
       * @return ein <code>int</code>-Wert
       */
      public int getLastBallUpdate() {return (time-ball.lastUpdate());}
      
      /**
       * ein mglicher Pass-Partner (= Spieler des eigenen Teams, der am nchsten zum Spieler steht
       * @return ein Objekt vom Typ <code>PlayerPerception</code>
       * @see PlayerPerception
       */
      public PlayerPerception getPassTeammate() {return possiblePassTeammate;}
      /**
       * die zuknftige Position des Balls
       *
       * @param cycles ein <code>int</code>-Wert, der angibt, fr wieviele Server-Cycles vorausberechnet werden soll
       * @return ein <code>double[2]</code>-Feld; Format: [x,y] 
       */
      public double[] getPredictedBallPos(int cycles) {return ball.predictPos(cycles);}
      
      
      /**
       * die relative Ballrichtung
       *
       * @return ein <code>int</code>-Wert
       */
      public int getRelBallDirection() {return ball.relDirection();}
      /**
       * die relative Richtung des gegnerischen Tors
       *
       * @return ein <code>int</code>-Wert
       */
      public int getRelOpGoalDirection() {if((opGoal.direction()-absPlayerAngle<=180))
      											return (opGoal.direction()-absPlayerAngle);
      									  else return (absPlayerAngle-opGoal.direction());}
      
      /**
       * die relative Richtung des eigenen Tors
       *
       * @return ein <code>int</code>-Wert
       */
      public int getRelOwnGoalDirection() {return (ownGoal.direction()-absPlayerAngle);}
      
      /**
       * Spielstand
       *
       * @return ein <code>int[2]</code>-Feld;
       * @return das erste Element sind die eigenen Tore, das zweite gegnerische Tore
       */
      public int[] getScore() {return score;}
      /**
       * die Liste der Spieler, die gesehen werden
       *
       * @return eine <code>Enumeration</code>, die eine Liste von <code>PlayerPerceptions</code> enthlt.
       * @see PlayerPerception
       */
      public Enumeration getSeenPlayerList() {return seenPlayers.elements();}
      /**
       * die Anfangs-X-Position des Spielers
       *
       * @return ein <code>double</code>-Wert
       * @see Positions
       */
      public double getStartX() {return Positions.ownKickOffX[unum-1];}
      /**
       * die Anfangs-Y-Position des Spielers
       *
       * @return ein <code>double</code>-Wert
       * @see Positions
       */
	  public double getStartY() {return Positions.ownKickOffY[unum-1];}	
      /**
       * Stamina (Ausdauer)
       * 
       * @return ein <code>double</code>-Wert
       * @return zwischen 0 und 4000
       */
      public double getStamina() {return stamina;}
      /**
        * die eigene Hlfte
        *
        * @return ein <code>String</code>;mgliche Werte sind "l" oder "r"
        */
      public String getSide() {return side;}
      /**
       * Teamname
       *
       * @return einen <code>String</code>;
       */
      public String getTeamName() {return teamName;}   
      
      /** 
       * Spielzeit
       *
       * @return ein <code>int</code> 
       */
      public int getTime() {return time;}
      /**
       * die Trikotnummer
       *
       * @return ein <code>int</code> 
       * @return mgliche Werte sind 1-11
       */
      public int getUnum() {return unum;}
      
      /**
       * Geschwindigkeit des Spielers
       *
       * @return ein <code>double</code>-Wert; 
       * @return mgliche Werte: 0 bis 1.0
       */
      public double getVelocity() {return velocity;}
      
      /**
       * die absolute Blickrichtung (Richtung des Kopfes)
       * 
       * @return ein <code>int</code>-Wert zwischen -180 und 180
       */
      public int getViewDirection() {return absPlayerAngle;}
      
      
      /**
       * Qualitt der aktuellen "change_view"-Einstellung
       * @return ein <code>String</code>; mgliche Werte sind "high","normal"
       */
      
      public String getViewQuality() {return viewQuality;}
      /**
       * Breite des Blickwinkels
       * @return ein <code>String</code>; mgliche Werte sind "wide","normal","narrow"
       */
      
      public String getViewWidth() {return viewWidth;}
  	  
      /**
       * ist Spieler der nchste Spieler des eigenen Teams zum Ball?
       * @return einen <code>boolean</code>-Wert der true ist, falls Spieler der nchste Spieler ist, ansonsten false.
       * @return Falls der Spieler keine anderen Spieler des eigenen Teams sieht, ist dieser Wert true
       */
      public boolean isClosestPlayerToBall() {return closestPlayerToBall;}	
      /**
   	   * ist der Spieler der nchste Spieler zur zuknftigen Position des Balls?
   	   * @param cycles ein <code>int</code>-Wert, der angibt, wie weit die Vorausberechnung erfolgen soll
   	   * @return ein <code>boolean</code>, der true ist, falls der Spieler der nchste Spieler zur Position ist, ansonsten false
   	   */
   	  public boolean isClosestPlayerToPredBall(int cycles) {
   	  			double[] predBall = ball.predictPos(cycles);
   	  			double ownDistToPredBall=Math.sqrt((playerPos[0]-predBall[0])*(playerPos[0]-predBall[0])+
        							(playerPos[1]-predBall[1])*(playerPos[1]-predBall[1]));
   	  			double teammatePredBallDist=5000;
   	  			
   	  			for(Enumeration e=seenPlayers.elements();e.hasMoreElements() ;) {
        			PlayerPerception playPer = (PlayerPerception)e.nextElement();
        			if(playPer.teamName().equals(teamName)) {
        					double distToPredBall=Math.sqrt((playPer.absX()-predBall[0])*(playPer.absX()-predBall[0])+
        							(playPer.absY()-predBall[1])*(playPer.absY()-predBall[1]));
        					if(distToPredBall<teammatePredBallDist);
        						teammatePredBallDist=distToPredBall;
        			}
        		}
        		if(ownDistToPredBall<teammatePredBallDist)
        			return true;
        		else
        			return false;
        		
        	}
      /**
       * ist Spieler der nchste Spieler des eigenen Teams zum gegnerischen Tor?
       * @return einen <code>boolean</code>-Wert der true ist, falls Spieler der nchste Spieler ist, ansonsten false
       * @return Falls der Spieler keine anderen Spieler des eigenen Teams sieht, ist dieser Wert true
       */
      public boolean isClosestPlayerToOpGoal() {return closestPlayerToOpGoal;}	
      
      /**
       * sieht der Spieler das eigene Tor?
       *
       * @return ein <code>boolean</code>-Wert, der true ist, falls der Spieler das Tor sieht, ansonsten false
       */
      public boolean ownGoalInSight() {return ownGoal.inSight;}
      /**
       * sieht der Spieler das gegnerische Tor?
       *
       * @return ein <code>boolean</code>-Wert, der true ist, falls der Spieler das Tor sieht, ansonsten false
       */
      public boolean opGoalInSight() {return opGoal.inSight;}
     
	  
      
      /**
       * der Spieler des eigenen Teams, der am nchsten zum gegnerischen Tor steht 
       *
       * @return ein Objekt vom Typ <code>PlayerPerception</code> 
       * @return Falls der Spieler selbst der nchste Spieler zum Tor ist,ist dieser Wert nicht sinnvoll.
       * @see PlayerPerception
       */
      public PlayerPerception getClosestTeammateToOpGoal() {return closestTeammateToOpGoal;}
      
      
   	  
   	  
   	  
   	     	  
}
